#
# Copyright (c) 2011-2014 Red Hat, Inc. <http://www.redhat.com>
# This file is part of GlusterFS.

# This file is licensed to you under your choice of the GNU Lesser
# General Public License, version 3 or any later version (LGPLv3 or
# later), or the GNU General Public License, version 2 (GPLv2), in all
# cases as published by the Free Software Foundation.
#

from __future__ import print_function
import os
import sys
import stat
import time
import struct
import random
import libcxattr

from errno import EEXIST

Xattr = libcxattr.Xattr()

def umask():
    return os.umask(0)

def _fmt_mknod(l):
    return "!II%dsI%dsIII" % (37, l+1)

def _fmt_mkdir(l):
    return "!II%dsI%dsII" % (37, l+1)

def _fmt_symlink(l1, l2):
    return "!II%dsI%ds%ds" % (37, l1+1, l2+1)


if sys.version_info > (3,):
    def entry_pack_reg(gf, bn, mo, uid, gid):
        bn_encoded = bn.encode()
        blen = len(bn_encoded)
        return struct.pack(_fmt_mknod(blen),
                           uid, gid, gf.encode(), mo, bn_encoded,
                           stat.S_IMODE(mo), 0, umask())

    # mkdir
    def entry_pack_dir(gf, bn, mo, uid, gid):
        bn_encoded = bn.encode()
        blen = len(bn_encoded)
        return struct.pack(_fmt_mkdir(blen),
                           uid, gid, gf.encode(), mo, bn_encoded,
                           stat.S_IMODE(mo), umask())
    # symlink
    def entry_pack_symlink(gf, bn, lnk, st):
        bn_encoded = bn.encode()
        blen = len(bn_encoded)
        lnk_encoded = lnk.encode()
        llen = len(lnk_encoded)
        return struct.pack(_fmt_symlink(blen, llen),
                           st['uid'], st['gid'],
                           gf.encode(), st['mode'], bn_encoded,
                           lnk_encoded)

else:
    def entry_pack_reg(gf, bn, mo, uid, gid):
        blen = len(bn)
        return struct.pack(_fmt_mknod(blen),
                           uid, gid, gf, mo, bn,
                           stat.S_IMODE(mo), 0, umask())

    def entry_pack_dir(gf, bn, mo, uid, gid):
        blen = len(bn)
        return struct.pack(_fmt_mkdir(blen),
                           uid, gid, gf, mo, bn,
                           stat.S_IMODE(mo), umask())

    def entry_pack_symlink(gf, bn, lnk, mo, uid, gid):
        blen = len(bn)
        llen = len(lnk)
        return struct.pack(_fmt_symlink(blen, llen),
                           uid, gid, gf, mo, bn, lnk)

if __name__ == '__main__':
    if len(sys.argv) < 9:
        print(("USAGE: %s <mount> <pargfid|ROOT> <filename> <GFID> <file type>"
              " <uid> <gid> <file permission(octal str)>" % (sys.argv[0])))
        sys.exit(-1) # nothing to do
    mtpt       = sys.argv[1]
    pargfid    = sys.argv[2]
    fname      = sys.argv[3]
    randomgfid = sys.argv[4]
    ftype      = sys.argv[5]
    uid        = int(sys.argv[6])
    gid        = int(sys.argv[7])
    perm       = int(sys.argv[8], 8)

    os.chdir(mtpt)
    if pargfid == 'ROOT':
        pargfid = '.gfid/00000000-0000-0000-0000-000000000001'
    else:
        pargfid = '.gfid/' + pargfid

    blob = None

    # entry op: use non-zero uid/gid (to catch gfid-access xlator bugs)
    if ftype == 'file':
        mode = stat.S_IFREG | perm
        blob = entry_pack_reg(randomgfid, fname, mode, uid, gid)
    elif ftype =='dir':
        mode = stat.S_IFDIR | perm
        blob = entry_pack_dir(randomgfid, fname, mode, uid, gid)
    else: # not yet...
        sys.exit(-1)

    if blob == None:
        sys.exit(-1)
    try:
        Xattr.lsetxattr(pargfid, 'glusterfs.gfid.newfile', blob)
    except OSError:
        ex = sys.exc_info()[1]
        if not ex.errno in [EEXIST]:
            raise
            sys.exit(-1)
    print("File creation OK")
    sys.exit(0)



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
#  Copyright (c) 2016 Red Hat, Inc. <http://www.redhat.com>
#  This file is part of GlusterFS.
#
#  This file is licensed to you under your choice of the GNU Lesser
#  General Public License, version 3 or any later version (LGPLv3 or
#  later), or the GNU General Public License, version 2 (GPLv2), in all
#  cases as published by the Free Software Foundation.
#

from argparse import ArgumentParser
import time
import os
from errno import EEXIST
import sys
import logging

from logutils import setup_logging
import gsyncdconfig as gconf
from rconf import rconf
import subcmds
from conf import GLUSTERD_WORKDIR, GLUSTERFS_CONFDIR, GCONF_VERSION
from syncdutils import (set_term_handler, finalize, lf,
                        log_raise_exception, FreeObject, escape)
import argsupgrade


GSYNCD_VERSION = "gsyncd.py %s.0" % GCONF_VERSION


def main():
    rconf.starttime = time.time()

    # If old Glusterd sends commands in old format, below function
    # converts the sys.argv to new format. This conversion is added
    # temporarily for backward compatibility. This can be removed
    # once integrated with Glusterd2
    # This modifies sys.argv globally, so rest of the code works as usual
    argsupgrade.upgrade()

    # Default argparse version handler prints to stderr, which is fixed in
    # 3.x series but not in 2.x, using custom parser to fix this issue
    if "--version" in sys.argv:
        print(GSYNCD_VERSION)
        sys.exit(0)

    parser = ArgumentParser()
    parser.add_argument("--inet6", action="store_true")
    sp = parser.add_subparsers(dest="subcmd")

    # Monitor Status File update
    p = sp.add_parser("monitor-status")
    p.add_argument("primary", help="Primary Volume Name")
    p.add_argument("secondary", help="Secondary details user@host::vol format")
    p.add_argument("status", help="Update Monitor Status")
    p.add_argument("-c", "--config-file", help="Config File")
    p.add_argument("--debug", action="store_true")

    # Monitor
    p = sp.add_parser("monitor")
    p.add_argument("primary", help="Primary Volume Name")
    p.add_argument("secondary", help="Secondary details user@host::vol format")
    p.add_argument("-c", "--config-file", help="Config File")
    p.add_argument("--pause-on-start",
                   action="store_true",
                   help="Start with Paused state")
    p.add_argument("--local-node-id", help="Local Node ID")
    p.add_argument("--debug", action="store_true")
    p.add_argument("--use-gconf-volinfo", action="store_true")

    # Worker
    p = sp.add_parser("worker")
    p.add_argument("primary", help="Primary Volume Name")
    p.add_argument("secondary", help="Secondary details user@host::vol format")
    p.add_argument("--local-path", help="Local Brick Path")
    p.add_argument("--feedback-fd", type=int,
                   help="feedback fd between monitor and worker")
    p.add_argument("--local-node", help="Local primary node")
    p.add_argument("--local-node-id", help="Local Node ID")
    p.add_argument("--subvol-num", type=int, help="Subvolume number")
    p.add_argument("--is-hottier", action="store_true",
                   help="Is this brick part of hot tier")
    p.add_argument("--resource-remote",
                   help="Remote node to connect to Secondary Volume")
    p.add_argument("--resource-remote-id",
                   help="Remote node ID to connect to Secondary Volume")
    p.add_argument("--secondary-id", help="Secondary Volume ID")
    p.add_argument("-c", "--config-file", help="Config File")
    p.add_argument("--debug", action="store_true")

    # Secondary
    p = sp.add_parser("secondary")
    p.add_argument("primary", help="Primary Volume Name")
    p.add_argument("secondary", help="Secondary details user@host::vol format")
    p.add_argument("--session-owner")
    p.add_argument("--primary-brick",
                   help="Primary brick which is connected to the Secondary")
    p.add_argument("--primary-node",
                   help="Primary node which is connected to the Secondary")
    p.add_argument("--primary-node-id",
                   help="Primary node ID which is connected to the Secondary")
    p.add_argument("--local-node", help="Local Secondary node")
    p.add_argument("--local-node-id", help="Local Secondary ID")
    p.add_argument("-c", "--config-file", help="Config File")
    p.add_argument("--debug", action="store_true")

    # All configurations which are configured via "secondary-" options
    # DO NOT add default values for these configurations, default values
    # will be picked from template config file
    p.add_argument("--secondary-timeout", type=int,
                   help="Timeout to end gsyncd at Secondary side")
    p.add_argument("--use-rsync-xattrs", action="store_true")
    p.add_argument("--secondary-log-level", help="Secondary Gsyncd Log level")
    p.add_argument("--secondary-gluster-log-level",
                   help="Secondary Gluster mount Log level")
    p.add_argument("--secondary-gluster-command-dir",
                   help="Directory where Gluster binaries exist on secondary")
    p.add_argument("--secondary-access-mount", action="store_true",
                   help="Do not lazy umount the secondary volume")
    p.add_argument("--primary-dist-count", type=int,
                   help="Primary Distribution count")

    # Status
    p = sp.add_parser("status")
    p.add_argument("primary", help="Primary Volume Name")
    p.add_argument("secondary", help="Secondary")
    p.add_argument("-c", "--config-file", help="Config File")
    p.add_argument("--local-path", help="Local Brick Path")
    p.add_argument("--debug", action="store_true")
    p.add_argument("--json", action="store_true")

    # Config-check
    p = sp.add_parser("config-check")
    p.add_argument("name", help="Config Name")
    p.add_argument("--value", help="Config Value")
    p.add_argument("--debug", action="store_true")

    # Config-get
    p = sp.add_parser("config-get")
    p.add_argument("primary", help="Primary Volume Name")
    p.add_argument("secondary", help="Secondary")
    p.add_argument("--name", help="Config Name")
    p.add_argument("-c", "--config-file", help="Config File")
    p.add_argument("--debug", action="store_true")
    p.add_argument("--show-defaults", action="store_true")
    p.add_argument("--only-value", action="store_true")
    p.add_argument("--use-underscore", action="store_true")
    p.add_argument("--json", action="store_true")

    # Config-set
    p = sp.add_parser("config-set")
    p.add_argument("primary", help="Primary Volume Name")
    p.add_argument("secondary", help="Secondary")
    p.add_argument("-n", "--name", help="Config Name")
    p.add_argument("-v", "--value", help="Config Value")
    p.add_argument("-c", "--config-file", help="Config File")
    p.add_argument("--debug", action="store_true")

    # Config-reset
    p = sp.add_parser("config-reset")
    p.add_argument("primary", help="Primary Volume Name")
    p.add_argument("secondary", help="Secondary")
    p.add_argument("name", help="Config Name")
    p.add_argument("-c", "--config-file", help="Config File")
    p.add_argument("--debug", action="store_true")

    # voluuidget
    p = sp.add_parser("voluuidget")
    p.add_argument("host", help="Hostname")
    p.add_argument("volname", help="Volume Name")
    p.add_argument("--debug", action="store_true")

    # Delete
    p = sp.add_parser("delete")
    p.add_argument("primary", help="Primary Volume Name")
    p.add_argument("secondary", help="Secondary")
    p.add_argument("-c", "--config-file", help="Config File")
    p.add_argument('--path', dest='paths', action="append")
    p.add_argument("--reset-sync-time", action="store_true",
                   help="Reset Sync Time")
    p.add_argument("--debug", action="store_true")

    # Parse arguments
    args = parser.parse_args()

    # Extra template values, All arguments are already part of template
    # variables, use this for adding extra variables
    extra_tmpl_args = {}

    # Add First/Primary Secondary host, user and volume
    if getattr(args, "secondary", None) is not None:
        hostdata, secondaryvol = args.secondary.split("::")
        hostdata = hostdata.split("@")
        secondaryhost = hostdata[-1]
        secondaryuser = "root"
        if len(hostdata) == 2:
            secondaryuser = hostdata[0]
        extra_tmpl_args["primary_secondary_host"] = secondaryhost
        extra_tmpl_args["secondaryuser"] = secondaryuser
        extra_tmpl_args["secondaryvol"] = secondaryvol

    # Add Bricks encoded path
    if getattr(args, "local_path", None) is not None:
        extra_tmpl_args["local_id"] = escape(args.local_path)

    # Add Primary Bricks encoded path(For Secondary)
    if getattr(args, "primary_brick", None) is not None:
        extra_tmpl_args["primary_brick_id"] = escape(args.primary_brick)

    # Load configurations
    config_file = getattr(args, "config_file", None)

    # Subcmd accepts config file argument but not passed
    # Set default path for config file in that case
    # If an subcmd accepts config file then it also accepts
    # primary and Secondary arguments.
    if config_file is None and hasattr(args, "config_file") \
        and args.subcmd != "secondary":
        config_file = "%s/geo-replication/%s_%s_%s/gsyncd.conf" % (
            GLUSTERD_WORKDIR,
            args.primary,
            extra_tmpl_args["primary_secondary_host"],
            extra_tmpl_args["secondaryvol"])

    # If Config file path not exists, log error and continue using default conf
    config_file_error_msg = None
    if config_file is not None and not os.path.exists(config_file):
        # Logging not yet initialized, create the error message to
        # log later and reset the config_file to None
        config_file_error_msg = lf(
            "Session config file not exists, using the default config",
            path=config_file)
        config_file = None

    rconf.config_file = config_file

    # Override gconf values from argument values only if it is secondary gsyncd
    override_from_args = False
    if args.subcmd == "secondary":
        override_from_args = True

    if config_file is not None and \
       args.subcmd in ["monitor", "config-get", "config-set", "config-reset"]:
        ret = gconf.is_config_file_old(config_file, args.primary, extra_tmpl_args["secondaryvol"])
        if ret is not None:
           gconf.config_upgrade(config_file, ret)

    # Load Config file
    gconf.load(GLUSTERFS_CONFDIR + "/gsyncd.conf",
               config_file,
               vars(args),
               extra_tmpl_args,
               override_from_args)

    # Default label to print in log file
    label = args.subcmd
    if args.subcmd in ("worker"):
        # If Worker, then add brick path also to label
        label = "%s %s" % (args.subcmd, args.local_path)
    elif args.subcmd == "secondary":
        # If Secondary add Primary node and Brick details
        label = "%s %s%s" % (args.subcmd, args.primary_node, args.primary_brick)

    # Setup Logger
    # Default log file
    log_file = gconf.get("cli-log-file")
    log_level = gconf.get("cli-log-level")
    if getattr(args, "primary", None) is not None and \
       getattr(args, "secondary", None) is not None:
        log_file = gconf.get("log-file")
        log_level = gconf.get("log-level")

    # Use different log file location for Secondary log file
    if args.subcmd == "secondary":
        log_file = gconf.get("secondary-log-file")
        log_level = gconf.get("secondary-log-level")

    if args.debug:
        log_file = "-"
        log_level = "DEBUG"

    # Create Logdir if not exists
    try:
        if log_file != "-":
            os.mkdir(os.path.dirname(log_file))
    except OSError as e:
        if e.errno != EEXIST:
            raise

    setup_logging(
        log_file=log_file,
        level=log_level,
        label=label
    )

    if config_file_error_msg is not None:
        logging.warn(config_file_error_msg)

    # Log message for loaded config file
    if config_file is not None:
        logging.debug(lf("Using session config file", path=config_file))

    set_term_handler()
    excont = FreeObject(exval=0)

    # Gets the function name based on the input argument. For example
    # if subcommand passed as argument is monitor then it looks for
    # function with name "subcmd_monitor" in subcmds file
    func = getattr(subcmds, "subcmd_" + args.subcmd.replace("-", "_"), None)

    try:
        try:
            if func is not None:
                rconf.args = args
                func(args)
        except:
            log_raise_exception(excont)
    finally:
        finalize(exval=excont.exval)


if __name__ == "__main__":
    main()



#!/usr/bin/python3
# -*- coding: utf-8 -*-

# Copyright (c) 2015 Red Hat, Inc. <http://www.redhat.com/>
# This file is part of GlusterFS.
#
# This file is licensed to you under your choice of the GNU Lesser
# General Public License, version 3 or any later version (LGPLv3 or
# later), or the GNU General Public License, version 2 (GPLv2), in all
# cases as published by the Free Software Foundation.

import shutil
import sys
import os
import logging
from argparse import ArgumentParser, RawDescriptionHelpFormatter
try:
    import urllib.parse as urllib
except ImportError:
    import urllib
from errno import ENOTEMPTY

from utils import setup_logger, mkdirp, handle_rm_error
import conf

logger = logging.getLogger()


def mode_cleanup(args):
    working_dir = os.path.join(conf.get_opt("working_dir"),
                               args.session,
                               args.volume,
                               args.tmpfilename)

    mkdirp(os.path.join(conf.get_opt("log_dir"), args.session, args.volume),
           exit_on_err=True)
    log_file = os.path.join(conf.get_opt("log_dir"),
                            args.session,
                            args.volume,
                            "changelog.log")

    setup_logger(logger, log_file)

    try:
        shutil.rmtree(working_dir, onerror=handle_rm_error)
    except (OSError, IOError) as e:
        logger.error("Failed to delete working directory: %s" % e)
        sys.exit(1)


def mode_create(args):
    session_dir = os.path.join(conf.get_opt("session_dir"),
                               args.session)
    status_file = os.path.join(session_dir, args.volume,
                     "%s.status" % urllib.quote_plus(args.brick))

    mkdirp(os.path.join(session_dir, args.volume), exit_on_err=True,
           logger=logger)

    if not os.path.exists(status_file) or args.reset_session_time:
        with open(status_file, "w") as f:
            f.write(args.time_to_update)

    sys.exit(0)


def mode_post(args):
    session_dir = os.path.join(conf.get_opt("session_dir"), args.session)
    status_file = os.path.join(session_dir, args.volume,
                     "%s.status" % urllib.quote_plus(args.brick))

    mkdirp(os.path.join(session_dir, args.volume), exit_on_err=True,
           logger=logger)
    status_file_pre = status_file + ".pre"

    if os.path.exists(status_file_pre):
        os.rename(status_file_pre, status_file)
        sys.exit(0)


def mode_delete(args):
    session_dir = os.path.join(conf.get_opt("session_dir"),
                               args.session)
    shutil.rmtree(os.path.join(session_dir, args.volume),
                  onerror=handle_rm_error)

    # If the session contains only this volume, then cleanup the
    # session directory. If a session contains multiple volumes
    # then os.rmdir will fail with ENOTEMPTY
    try:
        os.rmdir(session_dir)
    except OSError as e:
        if not e.errno == ENOTEMPTY:
            logger.warn("Failed to delete session directory: %s" % e)


def _get_args():
    parser = ArgumentParser(formatter_class=RawDescriptionHelpFormatter,
                            description="Node Agent")
    subparsers = parser.add_subparsers(dest="mode")

    parser_cleanup = subparsers.add_parser('cleanup')
    parser_cleanup.add_argument("session", help="Session Name")
    parser_cleanup.add_argument("volume", help="Volume Name")
    parser_cleanup.add_argument("tmpfilename", help="Temporary File Name")
    parser_cleanup.add_argument("--debug", help="Debug", action="store_true")

    parser_session_create = subparsers.add_parser('create')
    parser_session_create.add_argument("session", help="Session Name")
    parser_session_create.add_argument("volume", help="Volume Name")
    parser_session_create.add_argument("brick", help="Brick Path")
    parser_session_create.add_argument("time_to_update", help="Time to Update")
    parser_session_create.add_argument("--reset-session-time",
                                       help="Reset Session Time",
                                       action="store_true")
    parser_session_create.add_argument("--debug", help="Debug",
                                       action="store_true")

    parser_post = subparsers.add_parser('post')
    parser_post.add_argument("session", help="Session Name")
    parser_post.add_argument("volume", help="Volume Name")
    parser_post.add_argument("brick", help="Brick Path")
    parser_post.add_argument("--debug", help="Debug",
                             action="store_true")

    parser_delete = subparsers.add_parser('delete')
    parser_delete.add_argument("session", help="Session Name")
    parser_delete.add_argument("volume", help="Volume Name")
    parser_delete.add_argument("--debug", help="Debug",
                               action="store_true")
    return parser.parse_args()


if __name__ == "__main__":
    args = _get_args()

    # globals() will have all the functions already defined.
    # mode_<args.mode> will be the function name to be called
    globals()["mode_" + args.mode](args)



#
# Copyright (c) 2018 Red Hat, Inc. <http://www.redhat.com>
# This file is part of GlusterFS.

# This file is licensed to you under your choice of the GNU Lesser
# General Public License, version 3 or any later version (LGPLv3 or
# later), or the GNU General Public License, version 2 (GPLv2), in all
# cases as published by the Free Software Foundation.
#

# All python2/python3 compatibility routines

import os
import sys
from ctypes import create_string_buffer

if sys.version_info >= (3,):

    # Raw conversion of bytearray to string. Used in the cases where
    # buffer is created by create_string_buffer which is a 8-bit char
    # array and passed to syscalls to fetch results. Using encode/decode
    # doesn't work as it converts to string altering the size.
    # def bytearray_to_str(byte_arr):
    def bytearray_to_str(byte_arr):
        return ''.join([chr(b) for b in byte_arr])

    def gf_create_string_buffer(size):
        return create_string_buffer(b'\0', size)

    def gfind_history_changelog(libgfc, changelog_path, start, end, num_parallel,
                                actual_end):
        return libgfc.gf_history_changelog(changelog_path.encode(), start, end, num_parallel,
                       actual_end)

    def gfind_changelog_register(libgfc, brick, path, log_file, log_level,
                                 retries):
        return libgfc.gf_changelog_register(brick.encode(), path.encode(), log_file.encode(),
                       log_level, retries)

    def gfind_history_changelog_done(libgfc, clfile):
        return libgfc.gf_history_changelog_done(clfile.encode())

    def gfind_write_row(f, row, field_separator, p_rep, row_2_rep):
        f.write(u"{0}{1}{2}{3}{4}\n".format(row,
                                            field_separator,
                                            p_rep,
                                            field_separator,
                                            row_2_rep))

    def gfind_write(f, row, field_separator, p_rep):
        f.write(u"{0}{1}{2}\n".format(row,
                                      field_separator,
                                      p_rep))


else:

    # Raw conversion of bytearray to string
    def bytearray_to_str(byte_arr):
        return byte_arr

    def gf_create_string_buffer(size):
        return create_string_buffer('\0', size)

    def gfind_history_changelog(libgfc, changelog_path, start, end, num_parallel,
                                actual_end):
        return libgfc.gf_history_changelog(changelog_path, start, end,
                                                    num_parallel, actual_end)

    def gfind_changelog_register(libgfc, brick, path, log_file, log_level,
                                 retries):
        return libgfc.gf_changelog_register(brick, path, log_file,
                                                     log_level, retries)

    def gfind_history_changelog_done(libgfc, clfile):
        return libgfc.gf_history_changelog_done(clfile)

    def gfind_write_row(f, row, field_separator, p_rep, row_2_rep):
        f.write(u"{0}{1}{2}{3}{4}\n".format(row,
                                            field_separator,
                                            p_rep,
                                            field_separator,
                                            row_2_rep).encode())

    def gfind_write(f, row, field_separator, p_rep):
        f.write(u"{0}{1}{2}\n".format(row,
                                      field_separator,
                                      p_rep).encode())



#!/usr/bin/python3

from __future__ import print_function
import os
import sys

curdir = os.path.dirname(sys.argv[0])
gendir = os.path.join(curdir, '../../../../libglusterfs/src')
sys.path.append(gendir)
from generator import ops, fop_subs, cbk_subs, generate

FD_DATA_MODIFYING_OP_FOP_TEMPLATE = """
int32_t
cs_@NAME@ (call_frame_t *frame, xlator_t *this,
           @LONG_ARGS@)
{
        int                         op_errno        = EINVAL ;
        cs_local_t                 *local           = NULL;
        int                         ret             = 0;
        cs_inode_ctx_t             *ctx             = NULL;
        gf_cs_obj_state             state           = -1;

        VALIDATE_OR_GOTO (frame, err);
        VALIDATE_OR_GOTO (this, err);
        VALIDATE_OR_GOTO (fd, err);

        local = cs_local_init (this, frame, NULL, fd, GF_FOP_@UPNAME@);
        if (!local) {

                gf_msg (this->name, GF_LOG_ERROR, 0, 0, "local init failed");
                op_errno = ENOMEM;
                goto err;
        }

        __cs_inode_ctx_get (this, fd->inode, &ctx);

        if (ctx)
                state = __cs_get_file_state (fd->inode, ctx);
        else
                state = GF_CS_LOCAL;

        xdata = xdata ? dict_ref (xdata) : dict_new ();

        if (!xdata) {
                gf_msg (this->name, GF_LOG_ERROR, 0, 0, "insufficient memory");
                op_errno = ENOMEM;
                goto err;
        }

        local->xattr_req = xdata;

        ret = dict_set_uint32 (local->xattr_req, GF_CS_OBJECT_STATUS, 1);
        if (ret) {
                gf_msg (this->name, GF_LOG_ERROR, 0, 0, "dict_set failed key:"
                        " %s", GF_CS_OBJECT_STATUS);
                goto err;
        }

        local->stub = fop_@NAME@_stub (frame, cs_resume_@NAME@,
                                       @SHORT_ARGS@);
        if (!local->stub) {
                gf_msg (this->name, GF_LOG_ERROR, 0, 0, "insufficient memory");
                op_errno = ENOMEM;
                goto err;
        }


        if (state == GF_CS_LOCAL) {
                STACK_WIND (frame, cs_@NAME@_cbk,
                            FIRST_CHILD(this), FIRST_CHILD(this)->fops->@NAME@,
                            @SHORT_ARGS@);
        } else {
                local->call_cnt++;
                ret = locate_and_execute (frame);
                if (ret) {
                        op_errno = ENOMEM;
                        goto err;
                }
        }

        return 0;

err:
        CS_STACK_UNWIND (@NAME@, frame, -1, op_errno, @CBK_ERROR_ARGS@);

        return 0;
}
"""

FD_DATA_MODIFYING_RESUME_OP_FOP_TEMPLATE = """
int32_t
cs_resume_@NAME@ (call_frame_t *frame, xlator_t *this,
                  @LONG_ARGS@)
{
        int              ret    = 0;

        ret = cs_resume_postprocess (this, frame, fd->inode);
        if (ret) {
                goto unwind;
        }

        cs_inodelk_unlock (frame);

        STACK_WIND (frame, cs_@NAME@_cbk,
                    FIRST_CHILD(this), FIRST_CHILD(this)->fops->@NAME@,
                    @SHORT_ARGS@);

        return 0;

unwind:

        cs_inodelk_unlock (frame);

        cs_common_cbk (frame);

        return 0;
}
"""
FD_DATA_MODIFYING_OP_FOP_CBK_TEMPLATE = """
int32_t
cs_@NAME@_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
               int32_t op_ret, int32_t op_errno,
               @LONG_ARGS@)
{
        cs_local_t      *local = NULL;
        int              ret = 0;
        uint64_t         val = 0;
        fd_t            *fd = NULL;

        local = frame->local;
        fd = local->fd;

        /* Do we need lock here? */
        local->call_cnt++;

        if (op_ret == -1) {
                ret = dict_get_uint64 (xdata, GF_CS_OBJECT_STATUS, &val);
                if (ret == 0) {
                        if (val == GF_CS_ERROR) {
                                gf_msg (this->name, GF_LOG_ERROR, 0, 0,
                                        "could not get file state, unwinding");
                                op_ret = -1;
                                op_errno = EIO;
                                goto unwind;
                        } else {
                                __cs_inode_ctx_update (this, fd->inode, val);
                                gf_msg (this->name, GF_LOG_INFO, 0, 0,
                                        " state = %" PRIu64, val);

                                if (local->call_cnt == 1 &&
                                    (val == GF_CS_REMOTE ||
                                     val == GF_CS_DOWNLOADING))  {
                                        gf_msg (this->name, GF_LOG_INFO, 0,
                                                0, " will repair and download "
                                                "the file, current state : %"
                                                PRIu64, val);
                                        goto repair;
                                } else {
                                        gf_msg (this->name, GF_LOG_ERROR, 0, 0,
                                                "second @NAME@, Unwinding");
                                        goto unwind;
                                }
                        }
                } else {
                        gf_msg (this->name, GF_LOG_ERROR, 0, 0, "file state "
                                "could not be figured, unwinding");
                        goto unwind;
                }
        } else {
                /* successful @NAME@ => file is local */
                __cs_inode_ctx_update (this, fd->inode, GF_CS_LOCAL);
                gf_msg (this->name, GF_LOG_INFO, 0, 0, "state : GF_CS_LOCAL"
                        ", @NAME@ successful");

                goto unwind;
        }

repair:
        ret = locate_and_execute (frame);
        if (ret) {
                goto unwind;
        }

        return 0;

unwind:
        CS_STACK_UNWIND (@NAME@, frame, op_ret, op_errno, @SHORT_ARGS@);

        return 0;
}
"""

LOC_STAT_OP_FOP_TEMPLATE = """
int32_t
cs_@NAME@ (call_frame_t *frame, xlator_t *this,
           @LONG_ARGS@)
{
        int              op_errno = EINVAL;
        cs_local_t      *local = NULL;
        int              ret   = 0;

        local = cs_local_init (this, frame, loc, NULL, GF_FOP_@UPNAME@);
        if (!local) {
                gf_msg (this->name, GF_LOG_ERROR, 0, 0, "local is NULL");
                op_errno = ENOMEM;
                goto err;
        }

        if (loc->inode->ia_type == IA_IFDIR)
                goto wind;

        xdata = xdata ? dict_ref (xdata) : dict_new ();

        if (!xdata) {
                gf_msg (this->name, GF_LOG_ERROR, 0, 0, "insufficient memory");
                op_errno = ENOMEM;
                goto err;
        }

        local->xattr_req = xdata;

        ret = dict_set_uint32 (local->xattr_req, GF_CS_OBJECT_STATUS, 1);
        if (ret) {
                gf_msg (this->name, GF_LOG_ERROR, 0, 0, "dict_set failed key:"
                        " %s", GF_CS_OBJECT_STATUS);
                goto err;
        }

wind:
        STACK_WIND (frame, cs_@NAME@_cbk, FIRST_CHILD(this),
                    FIRST_CHILD(this)->fops->@NAME@,
                    @SHORT_ARGS@);

        return 0;
err:
        CS_STACK_UNWIND (@NAME@, frame, -1, op_errno, @CBK_ERROR_ARGS@);

        return 0;
}
"""

LOC_STAT_OP_FOP_CBK_TEMPLATE = """
int32_t
cs_@NAME@_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
               int32_t op_ret, int32_t op_errno,
               @LONG_ARGS@)
{
        int              ret = 0;
        uint64_t         val = 0;
        loc_t           *loc = NULL;
        cs_local_t      *local = NULL;

        local = frame->local;

        loc = &local->loc;

        if (op_ret == 0) {
                ret = dict_get_uint64 (xdata, GF_CS_OBJECT_STATUS, &val);
                if (!ret) {
                        ret = __cs_inode_ctx_update (this, loc->inode, val);
                        if (ret) {
                                gf_msg (this->name, GF_LOG_ERROR, 0, 0,
                                        "ctx update failed");
                        }
                }
        } else {
                cs_inode_ctx_reset (this, loc->inode);
        }

        CS_STACK_UNWIND (@NAME@, frame, op_ret, op_errno, @SHORT_ARGS@);

        return 0;
}
"""

# All xlator FOPs are covered in the following section just to create a clarity
# The lists themselves are not used.
entry_ops = ['mknod', 'mkdir', 'unlink', 'rmdir', 'symlink', 'rename', 'link',
             'create']
special_ops = ['statfs', 'lookup', 'ipc', 'compound', 'icreate', 'namelink']
ignored_ops = ['getspec']
inode_ops = ['stat', 'readlink', 'truncate', 'open', 'setxattr', 'getxattr',
             'removexattr', 'opendir', 'access', 'inodelk', 'entrylk',
             'xattrop', 'setattr', 'lease', 'getactivelk', 'setactivelk',
             'discover']
fd_ops = ['readv', 'writev', 'flush', 'fsync', 'fsyncdir', 'ftruncate',
          'fstat', 'lk', 'readdir', 'finodelk', 'fentrylk', 'fxattrop',
          'fsetxattr', 'fgetxattr', 'rchecksum', 'fsetattr', 'readdirp',
          'fremovexattr', 'fallocate', 'discard', 'zerofill', 'seek']


# These are the current actual lists used to generate the code

# The following list contains fops which are fd based that modifies data
fd_data_modify_op_fop_template = ['writev', 'flush', 'fsync',
                                  'ftruncate', 'rchecksum', 'fallocate',
                                  'discard', 'zerofill', 'seek']

# The following list contains fops which are entry based that does not change
# data
loc_stat_op_fop_template = ['lookup', 'stat', 'discover', 'access', 'setattr',
                            'getattr']

# These fops need a separate implementation
special_fops = ['statfs', 'setxattr', 'unlink', 'getxattr',
                'truncate', 'fstat', 'readv', 'readdirp']

def gen_defaults():
    for name in ops:
        if name in fd_data_modify_op_fop_template:
            print(generate(FD_DATA_MODIFYING_OP_FOP_CBK_TEMPLATE, name, cbk_subs))
            print(generate(FD_DATA_MODIFYING_RESUME_OP_FOP_TEMPLATE, name, fop_subs))
            print(generate(FD_DATA_MODIFYING_OP_FOP_TEMPLATE, name, fop_subs))
        elif name in loc_stat_op_fop_template:
            print(generate(LOC_STAT_OP_FOP_CBK_TEMPLATE, name, cbk_subs))
            print(generate(LOC_STAT_OP_FOP_TEMPLATE, name, fop_subs))

for l in open(sys.argv[1], 'r').readlines():
    if l.find('#pragma generate') != -1:
        print("/* BEGIN GENERATED CODE - DO NOT MODIFY */")
        gen_defaults()
        print("/* END GENERATED CODE */")
    else:
        print(l[:-1])



#!/usr/bin/python3
#
# Copyright (c) 2015 Red Hat, Inc. <http://www.redhat.com>
# This file is part of GlusterFS.
#
# This file is licensed to you under your choice of the GNU Lesser
# General Public License, version 3 or any later version (LGPLv3 or
# later), or the GNU General Public License, version 2 (GPLv2), in all
# cases as published by the Free Software Foundation.

from __future__ import print_function
import subprocess
import os
import os.path
import logging
import argparse
import fcntl
import logging.handlers
import sys
import shutil
from errno import EEXIST
from conf import GLUSTERFS_LIBEXECDIR
sys.path.insert(1, GLUSTERFS_LIBEXECDIR)

EVENTS_ENABLED = True
try:
    from events.eventtypes import SNAPSHOT_SCHEDULER_INITIALISED \
                         as EVENT_SNAPSHOT_SCHEDULER_INITIALISED
    from events.eventtypes import SNAPSHOT_SCHEDULER_INIT_FAILED \
                         as EVENT_SNAPSHOT_SCHEDULER_INIT_FAILED
    from events.eventtypes import SNAPSHOT_SCHEDULER_DISABLED \
                         as EVENT_SNAPSHOT_SCHEDULER_DISABLED
    from events.eventtypes import SNAPSHOT_SCHEDULER_DISABLE_FAILED \
                         as EVENT_SNAPSHOT_SCHEDULER_DISABLE_FAILED
    from events.eventtypes import SNAPSHOT_SCHEDULER_ENABLED \
                         as EVENT_SNAPSHOT_SCHEDULER_ENABLED
    from events.eventtypes import SNAPSHOT_SCHEDULER_ENABLE_FAILED \
                         as EVENT_SNAPSHOT_SCHEDULER_ENABLE_FAILED
    from events.eventtypes import SNAPSHOT_SCHEDULER_SCHEDULE_ADDED \
                         as EVENT_SNAPSHOT_SCHEDULER_SCHEDULE_ADDED
    from events.eventtypes import SNAPSHOT_SCHEDULER_SCHEDULE_ADD_FAILED \
                         as EVENT_SNAPSHOT_SCHEDULER_SCHEDULE_ADD_FAILED
    from events.eventtypes import SNAPSHOT_SCHEDULER_SCHEDULE_DELETED \
                         as EVENT_SNAPSHOT_SCHEDULER_SCHEDULE_DELETED
    from events.eventtypes import SNAPSHOT_SCHEDULER_SCHEDULE_DELETE_FAILED \
                         as EVENT_SNAPSHOT_SCHEDULER_SCHEDULE_DELETE_FAILED
    from events.eventtypes import SNAPSHOT_SCHEDULER_SCHEDULE_EDITED \
                         as EVENT_SNAPSHOT_SCHEDULER_SCHEDULE_EDITED
    from events.eventtypes import SNAPSHOT_SCHEDULER_SCHEDULE_EDIT_FAILED \
                         as EVENT_SNAPSHOT_SCHEDULER_SCHEDULE_EDIT_FAILED
except ImportError:
    # Events APIs not installed, dummy eventtypes with None
    EVENTS_ENABLED = False
    EVENT_SNAPSHOT_SCHEDULER_INITIALISED = None
    EVENT_SNAPSHOT_SCHEDULER_INIT_FAILED = None
    EVENT_SNAPSHOT_SCHEDULER_DISABLED = None
    EVENT_SNAPSHOT_SCHEDULER_DISABLE_FAILED = None
    EVENT_SNAPSHOT_SCHEDULER_ENABLED = None
    EVENT_SNAPSHOT_SCHEDULER_ENABLE_FAILED = None
    EVENT_SNAPSHOT_SCHEDULER_SCHEDULE_ADDED = None
    EVENT_SNAPSHOT_SCHEDULER_SCHEDULE_ADD_FAILED = None
    EVENT_SNAPSHOT_SCHEDULER_SCHEDULE_DELETED = None
    EVENT_SNAPSHOT_SCHEDULER_SCHEDULE_DELETE_FAILED = None
    EVENT_SNAPSHOT_SCHEDULER_SCHEDULE_EDITED = None
    EVENT_SNAPSHOT_SCHEDULER_SCHEDULE_EDIT_FAILED = None

SCRIPT_NAME = "snap_scheduler"
scheduler_enabled = False
log = logging.getLogger(SCRIPT_NAME)
SHARED_STORAGE_DIR="/run/gluster/shared_storage"
GCRON_DISABLED = SHARED_STORAGE_DIR+"/snaps/gcron_disabled"
GCRON_ENABLED = SHARED_STORAGE_DIR+"/snaps/gcron_enabled"
GCRON_TASKS = SHARED_STORAGE_DIR+"/snaps/glusterfs_snap_cron_tasks"
GCRON_CROND_TASK = "/etc/cron.d/glusterfs_snap_cron_tasks"
LOCK_FILE_DIR = SHARED_STORAGE_DIR+"/snaps/lock_files/"
LOCK_FILE = LOCK_FILE_DIR+"lock_file"
TMP_FILE = SHARED_STORAGE_DIR+"/snaps/tmp_file"
GCRON_UPDATE_TASK = "/etc/cron.d/gcron_update_task"
CURRENT_SCHEDULER = SHARED_STORAGE_DIR+"/snaps/current_scheduler"
tasks = {}
longest_field = 12
current_scheduler = ""

INTERNAL_ERROR = 2
SHARED_STORAGE_DIR_DOESNT_EXIST = 3
SHARED_STORAGE_NOT_MOUNTED = 4
ANOTHER_TRANSACTION_IN_PROGRESS = 5
INIT_FAILED = 6
SCHEDULING_ALREADY_DISABLED = 7
SCHEDULING_ALREADY_ENABLED = 8
NODE_NOT_INITIALISED = 9
ANOTHER_SCHEDULER_ACTIVE = 10
JOB_ALREADY_EXISTS = 11
JOB_NOT_FOUND = 12
INVALID_JOBNAME = 13
INVALID_VOLNAME = 14
INVALID_SCHEDULE = 15
INVALID_ARG = 16
VOLUME_DOES_NOT_EXIST = 17

def print_error (error_num):
    if error_num == INTERNAL_ERROR:
        return "Internal Error"
    elif error_num == SHARED_STORAGE_DIR_DOESNT_EXIST:
        return "The shared storage directory ("+SHARED_STORAGE_DIR+")" \
               " does not exist."
    elif error_num == SHARED_STORAGE_NOT_MOUNTED:
        return "The shared storage directory ("+SHARED_STORAGE_DIR+")" \
               " is not mounted."
    elif error_num == ANOTHER_TRANSACTION_IN_PROGRESS:
        return "Another transaction is in progress."
    elif error_num == INIT_FAILED:
        return "Initialisation failed."
    elif error_num == SCHEDULING_ALREADY_DISABLED:
        return "Snapshot scheduler is already disabled."
    elif error_num == SCHEDULING_ALREADY_ENABLED:
        return "Snapshot scheduler is already enabled."
    elif error_num == NODE_NOT_INITIALISED:
        return "The node is not initialised."
    elif error_num == ANOTHER_SCHEDULER_ACTIVE:
        return "Another scheduler is active."
    elif error_num == JOB_ALREADY_EXISTS:
        return "The job already exists."
    elif error_num == JOB_NOT_FOUND:
        return "The job cannot be found."
    elif error_num == INVALID_JOBNAME:
        return "The job name is invalid."
    elif error_num == INVALID_VOLNAME:
        return "The volume name is invalid."
    elif error_num == INVALID_SCHEDULE:
        return "The schedule is invalid."
    elif error_num == INVALID_ARG:
        return "The argument is invalid."
    elif error_num == VOLUME_DOES_NOT_EXIST:
        return "The volume does not exist."

def output(msg):
    print("%s: %s" % (SCRIPT_NAME, msg))


def initLogger():
    log.setLevel(logging.DEBUG)
    logFormat = "[%(asctime)s %(filename)s:%(lineno)s %(funcName)s] "\
        "%(levelname)s %(message)s"
    formatter = logging.Formatter(logFormat)

    sh = logging.handlers.SysLogHandler()
    sh.setLevel(logging.ERROR)
    sh.setFormatter(formatter)

    process = subprocess.Popen(["gluster", "--print-logdir"],
                               stdout=subprocess.PIPE, universal_newlines=True)
    logfile = os.path.join(process.stdout.read()[:-1], SCRIPT_NAME + ".log")

    fh = logging.FileHandler(logfile)
    fh.setLevel(logging.DEBUG)
    fh.setFormatter(formatter)

    log.addHandler(sh)
    log.addHandler(fh)


def scheduler_status():
    ret = INTERNAL_ERROR
    global scheduler_enabled
    try:
        f = os.path.realpath(GCRON_TASKS)
        if f != os.path.realpath(GCRON_ENABLED) or not os.path.exists(GCRON_ENABLED):
            log.info("Snapshot scheduler is currently disabled.")
            scheduler_enabled = False
        else:
            log.info("Snapshot scheduler is currently enabled.")
            scheduler_enabled = True
        ret = 0
    except:
        log.error("Failed to enable snapshot scheduling. Error: "
                  "Failed to check the status of %s.", GCRON_DISABLED)

    return ret

def enable_scheduler():
    ret = scheduler_status()
    if ret == 0:
        if not scheduler_enabled:

            # Check if another scheduler is active.
            ret = get_current_scheduler()
            if ret == 0:
                if (current_scheduler != "none"):
                    print_str = "Failed to enable snapshot scheduling. " \
                                "Error: Another scheduler is active."
                    log.error(print_str)
                    output(print_str)
                    ret = ANOTHER_SCHEDULER_ACTIVE
                    return ret
            else:
                print_str = "Failed to get current scheduler info."
                log.error(print_str)
                output(print_str)
                return ret

            log.info("Enabling snapshot scheduler.")
            try:
                if os.path.exists(GCRON_DISABLED):
                    os.remove(GCRON_DISABLED)
                if os.path.lexists(GCRON_TASKS):
                    os.remove(GCRON_TASKS)
                try:
                    f = os.open(GCRON_ENABLED, os.O_CREAT | os.O_NONBLOCK,
                                0o644)
                    os.close(f)
                except OSError as e:
                    log.error("Failed to open %s. Error: %s.",
                              GCRON_ENABLED, e)
                    ret = INTERNAL_ERROR
                    return ret
                os.symlink(GCRON_ENABLED, GCRON_TASKS)
                update_current_scheduler("cli")
                log.info("Snapshot scheduling is enabled")
                output("Snapshot scheduling is enabled")
                ret = 0
            except OSError as e:
                print_str = ("Failed to enable snapshot scheduling."
                             "Error: {{}}" + e)
                log.error(print_str)
                output(print_str)
                ret = INTERNAL_ERROR
        else:
            print_str = "Failed to enable snapshot scheduling. " \
                        "Error: Snapshot scheduling is already enabled."
            log.error(print_str)
            output(print_str)
            ret = SCHEDULING_ALREADY_ENABLED
    else:
        print_str = "Failed to enable snapshot scheduling. " \
                    "Error: Failed to check scheduler status."
        log.error(print_str)
        output(print_str)

    return ret


def disable_scheduler():
    ret = scheduler_status()
    if ret == 0:
        if scheduler_enabled:
            log.info("Disabling snapshot scheduler.")
            try:
                # Check if another scheduler is active. If not, then
                # update current scheduler to "none". Else do nothing.
                ret = get_current_scheduler()
                if ret == 0:
                    if (current_scheduler == "cli"):
                        update_current_scheduler("none")
                else:
                    print_str = "Failed to disable snapshot scheduling. " \
                                "Error: Failed to get current scheduler info."
                    log.error(print_str)
                    output(print_str)
                    return ret

                if os.path.exists(GCRON_DISABLED):
                    os.remove(GCRON_DISABLED)
                if os.path.lexists(GCRON_TASKS):
                    os.remove(GCRON_TASKS)
                f = os.open(GCRON_DISABLED, os.O_CREAT, 0o644)
                os.close(f)
                os.symlink(GCRON_DISABLED, GCRON_TASKS)
                log.info("Snapshot scheduling is disabled")
                output("Snapshot scheduling is disabled")
                ret = 0
            except OSError as e:
                print_str = ("Failed to disable snapshot scheduling. Error: "
                             + e)
                log.error(print_str)
                output(print_str)
                ret = INTERNAL_ERROR
        else:
            print_str = "Failed to disable scheduling. " \
                        "Error: Snapshot scheduling is already disabled."
            log.error(print_str)
            output(print_str)
            ret = SCHEDULING_ALREADY_DISABLED
    else:
        print_str = "Failed to disable snapshot scheduling. " \
                    "Error: Failed to check scheduler status."
        log.error(print_str)
        output(print_str)
        ret = INTERNAL_ERROR

    return ret


def load_tasks_from_file():
    global tasks
    global longest_field
    try:
        with open(GCRON_ENABLED, 'r') as f:
            for line in f:
                line = line.rstrip('\n')
                if not line:
                    break
                line = line.split("gcron.py")
                schedule = line[0].split("root")[0].rstrip(' ')
                line = line[1].split(" ")
                volname = line[1]
                jobname = line[2]
                longest_field = max(longest_field, len(jobname), len(volname),
                                    len(schedule))
                tasks[jobname] = schedule+":"+volname
            f.close()
        ret = 0
    except IOError as e:
        log.error("Failed to open %s. Error: %s.", GCRON_ENABLED, e)
        ret = INTERNAL_ERROR

    return ret


def get_current_scheduler():
    global current_scheduler
    try:
        with open(CURRENT_SCHEDULER, 'r') as f:
            current_scheduler = f.readline().rstrip('\n')
            f.close()
        ret = 0
    except IOError as e:
        log.error("Failed to open %s. Error: %s.", CURRENT_SCHEDULER, e)
        ret = INTERNAL_ERROR

    return ret


def list_schedules():
    log.info("Listing snapshot schedules.")
    ret = load_tasks_from_file()
    if ret == 0:
        if len(tasks) == 0:
            output("No snapshots scheduled")
        else:
            jobname = "JOB_NAME".ljust(longest_field+5)
            schedule = "SCHEDULE".ljust(longest_field+5)
            operation = "OPERATION".ljust(longest_field+5)
            volname = "VOLUME NAME".ljust(longest_field+5)
            hyphens = "".ljust((longest_field+5) * 4, '-')
            print(jobname+schedule+operation+volname)
            print(hyphens)
            for key in sorted(tasks):
                jobname = key.ljust(longest_field+5)
                schedule = tasks[key].split(":")[0].ljust(
                           longest_field + 5)
                volname = tasks[key].split(":")[1].ljust(
                          longest_field + 5)
                operation = "Snapshot Create".ljust(longest_field+5)
                print(jobname+schedule+operation+volname)
            ret = 0
    else:
        print_str = "Failed to list snapshot schedules. " \
                    "Error: Failed to load tasks from "+GCRON_ENABLED
        log.error(print_str)
        output(print_str)

    return ret


def write_tasks_to_file():
    try:
        with open(TMP_FILE, "w", 0o644) as f:
            # If tasks is empty, just create an empty tmp file
            if len(tasks) != 0:
                for key in sorted(tasks):
                    jobname = key
                    schedule = tasks[key].split(":")[0]
                    volname = tasks[key].split(":")[1]
                    f.write("%s root PATH=$PATH:/usr/local/sbin:/usr/sbin "
                            "gcron.py %s %s\n" % (schedule, volname, jobname))
                f.write("\n")
                f.flush()
                os.fsync(f.fileno())
            f.close()
    except IOError as e:
        log.error("Failed to open %s. Error: %s.", TMP_FILE, e)
        ret = INTERNAL_ERROR
        return ret

    shutil.move(TMP_FILE, GCRON_ENABLED)
    ret = 0

    return ret

def update_current_scheduler(data):
    try:
        with open(TMP_FILE, "w", 0o644) as f:
            f.write("%s" % data)
            f.flush()
            os.fsync(f.fileno())
            f.close()
    except IOError as e:
        log.error("Failed to open %s. Error: %s.", TMP_FILE, e)
        ret = INTERNAL_ERROR
        return ret

    shutil.move(TMP_FILE, CURRENT_SCHEDULER)
    ret = 0

    return ret


def isVolumePresent(volname):
    success = False
    if volname == "":
        log.debug("No volname given")
        return success

    cli = ["gluster",
           "volume",
           "info",
           volname]
    log.debug("Running command '%s'", " ".join(cli))

    p = subprocess.Popen(cli, stdout=subprocess.PIPE,
                         stderr=subprocess.PIPE)
    out, err = p.communicate()
    rv = p.returncode

    log.debug("Command '%s' returned '%d'", " ".join(cli), rv)

    if rv:
        log.error("Command output:")
        log.error(err)
    else:
        success = True;

    return success


def add_schedules(jobname, schedule, volname):
    log.info("Adding snapshot schedules.")
    ret = load_tasks_from_file()
    if ret == 0:
        if jobname in tasks:
            print_str = ("%s already exists in schedule. Use "
                         "'edit' to modify %s" % (jobname, jobname))
            log.error(print_str)
            output(print_str)
            ret = JOB_ALREADY_EXISTS
        else:
            if not isVolumePresent(volname):
                print_str = ("Volume %s does not exist. Create %s and retry." %
                             (volname, volname))
                log.error(print_str)
                output(print_str)
                ret = VOLUME_DOES_NOT_EXIST
            else:
                tasks[jobname] = schedule + ":" + volname
                ret = write_tasks_to_file()
                if ret == 0:
                    # Create a LOCK_FILE for the job
                    job_lockfile = LOCK_FILE_DIR + jobname
                    try:
                        f = os.open(job_lockfile, os.O_CREAT | os.O_NONBLOCK,
                                    0o644)
                        os.close(f)
                    except OSError as e:
                        log.error("Failed to open %s. Error: %s.",
                                  job_lockfile, e)
                        ret = INTERNAL_ERROR
                        return ret
                    log.info("Successfully added snapshot schedule %s" %
                             jobname)
                    output("Successfully added snapshot schedule")
                    ret = 0
    else:
        print_str = "Failed to add snapshot schedule. " \
                    "Error: Failed to load tasks from "+GCRON_ENABLED
        log.error(print_str)
        output(print_str)

    return ret


def delete_schedules(jobname):
    log.info("Delete snapshot schedules.")
    ret = load_tasks_from_file()
    if ret == 0:
        if jobname in tasks:
            del tasks[jobname]
            ret = write_tasks_to_file()
            if ret == 0:
                # Delete the LOCK_FILE for the job
                job_lockfile = LOCK_FILE_DIR+jobname
                try:
                    os.remove(job_lockfile)
                except OSError as e:
                    log.error("Failed to open %s. Error: %s.",
                              job_lockfile, e)
                    ret = INTERNAL_ERROR
                    return ret
                log.info("Successfully deleted snapshot schedule %s"
                         % jobname)
                output("Successfully deleted snapshot schedule")
                ret = 0
        else:
            print_str = ("Failed to delete %s. Error: No such "
                         "job scheduled" % jobname)
            log.error(print_str)
            output(print_str)
            ret = JOB_NOT_FOUND
    else:
        print_str = "Failed to delete snapshot schedule. " \
                    "Error: Failed to load tasks from "+GCRON_ENABLED
        log.error(print_str)
        output(print_str)

    return ret


def edit_schedules(jobname, schedule, volname):
    log.info("Editing snapshot schedules.")
    ret = load_tasks_from_file()
    if ret == 0:
        if jobname in tasks:
            if not isVolumePresent(volname):
                print_str = ("Volume %s does not exist. Create %s and retry." %
                             (volname, volname))
                log.error(print_str)
                output(print_str)
                ret = VOLUME_DOES_NOT_EXIST
            else:
                tasks[jobname] = schedule+":"+volname
                ret = write_tasks_to_file()
                if ret == 0:
                    log.info("Successfully edited snapshot schedule %s" %
                             jobname)
                    output("Successfully edited snapshot schedule")
        else:
            print_str = ("Failed to edit %s. Error: No such "
                         "job scheduled" % jobname)
            log.error(print_str)
            output(print_str)
            ret = JOB_NOT_FOUND
    else:
        print_str = "Failed to edit snapshot schedule. " \
                    "Error: Failed to load tasks from "+GCRON_ENABLED
        log.error(print_str)
        output(print_str)

    return ret

def get_bool_val():
    getsebool_cli = ["getsebool",
                     "-a"]
    p1 = subprocess.Popen(getsebool_cli, stdout=subprocess.PIPE,
                          stderr=subprocess.PIPE)

    grep_cmd = ["grep",
                "cron_system_cronjob_use_shares"]
    p2 = subprocess.Popen(grep_cmd, stdin=p1.stdout,
                          stdout=subprocess.PIPE,
                          stderr=subprocess.PIPE)

    p1.stdout.close()
    output, err = p2.communicate()
    rv = p2.returncode

    if rv:
        log.error("Command output:")
        log.error(err)
        return -1

    bool_val = output.split()[2]
    log.debug("Bool value = '%s'", bool_val)

    return bool_val

def get_selinux_status():
    getenforce_cli = ["getenforce"]
    log.debug("Running command '%s'", " ".join(getenforce_cli))

    try:
        p1 = subprocess.Popen(getenforce_cli, stdout=subprocess.PIPE,
                              stderr=subprocess.PIPE)
    except OSError as oserr:
        log.error("Failed to run the command \"getenforce\". Error: %s" %\
                  oserr)
        return -1

    output, err = p1.communicate()
    rv = p1.returncode

    if rv:
        log.error("Command output:")
        log.error(err)
        return -1
    else:
        selinux_status=output.rstrip()
        log.debug("selinux status: %s", selinux_status)

    return selinux_status

def set_cronjob_user_share():
    selinux_status = get_selinux_status()
    if (selinux_status == -1):
        log.error("Failed to get selinux status")
        return -1
    elif (selinux_status == "Disabled"):
        return 0

    bool_val = get_bool_val()
    # In case of a failure (where the boolean value is not)
    # present in the system, we should not proceed further
    # We should only proceed when the value is "off"
    if (bool_val == -1 or bool_val != "off"):
        return 0

    setsebool_cli = ["setsebool", "-P",
                     "cron_system_cronjob_use_shares",
                     "on"]
    log.debug("Running command '%s'", " ".join(setsebool_cli))

    p1 = subprocess.Popen(setsebool_cli, stdout=subprocess.PIPE,
                          stderr=subprocess.PIPE)

    output, err = p1.communicate()
    rv = p1.returncode

    if rv:
        log.error("Command output:")
        log.error(err)
        return rv

    bool_val = get_bool_val()
    if (bool_val == "on"):
        return 0
    else:
        # In case of an error or if boolean is not on
        # we return a failure here
        return -1

def initialise_scheduler():
    ret = set_cronjob_user_share()
    if ret:
        log.error("Failed to set selinux boolean "
                  "cron_system_cronjob_use_shares to 'on'")
        return ret

    try:
        with open(TMP_FILE, "w+", 0o644) as f:
            updater = ("* * * * * root PATH=$PATH:/usr/local/sbin:"
                       "/usr/sbin gcron.py --update\n")
            f.write("%s\n" % updater)
            f.flush()
            os.fsync(f.fileno())
            f.close()
    except IOError as e:
        log.error("Failed to open %s. Error: %s.", TMP_FILE, e)
        ret = INIT_FAILED
        return ret

    shutil.move(TMP_FILE, GCRON_UPDATE_TASK)

    if not os.path.lexists(GCRON_TASKS):
        try:
            f = open(GCRON_TASKS, "w", 0o644)
            f.close()
        except IOError as e:
            log.error("Failed to open %s. Error: %s.", GCRON_TASKS, e)
            ret = INIT_FAILED
            return ret

    if os.path.lexists(GCRON_CROND_TASK):
        os.remove(GCRON_CROND_TASK)

    os.symlink(GCRON_TASKS, GCRON_CROND_TASK)

    log.info("Successfully initialised snapshot scheduler for this node")
    output("Successfully initialised snapshot scheduler for this node")
    gf_event (EVENT_SNAPSHOT_SCHEDULER_INITIALISED, status="Success")

    ret = 0
    return ret


def syntax_checker(args):
    if hasattr(args, 'jobname'):
        if (len(args.jobname.split()) != 1):
            output("Invalid Jobname. Jobname should not be empty and should not contain \" \" character.")
            ret = INVALID_JOBNAME
            return ret
        args.jobname=args.jobname.strip()

    if hasattr(args, 'volname'):
        if (len(args.volname.split()) != 1):
            output("Invalid Volname. Volname should not be empty and should not contain \" \" character.")
            ret = INVALID_VOLNAME
            return ret
        args.volname=args.volname.strip()

    if hasattr(args, 'schedule'):
        if (len(args.schedule.split()) != 5):
            output("Invalid Schedule. Please refer to the following for adding a valid cron schedule")
            print ("* * * * *")
            print ("| | | | |")
            print ("| | | | +---- Day of the Week   (range: 1-7, 1 standing for Monday)")
            print ("| | | +------ Month of the Year (range: 1-12)")
            print ("| | +-------- Day of the Month  (range: 1-31)")
            print ("| +---------- Hour              (range: 0-23)")
            print ("+------------ Minute            (range: 0-59)")
            ret = INVALID_SCHEDULE
            return ret

    ret = 0
    return ret


def perform_operation(args):
    if not os.path.exists(CURRENT_SCHEDULER):
        update_current_scheduler("none")

    # Initialise snapshot scheduler on local node
    if args.action == "init":
        ret = initialise_scheduler()
        if ret != 0:
            output("Failed to initialise snapshot scheduling")
            gf_event (EVENT_SNAPSHOT_SCHEDULER_INIT_FAILED,
                      error=print_error(ret))
        return ret

    # Disable snapshot scheduler
    if args.action == "disable_force":
        ret = disable_scheduler()
        if ret == 0:
            subprocess.Popen(["touch", "-h", GCRON_TASKS])
            gf_event (EVENT_SNAPSHOT_SCHEDULER_DISABLED,
                      status="Successfully Disabled")
        else:
            gf_event (EVENT_SNAPSHOT_SCHEDULER_DISABLE_FAILED,
                      error=print_error(ret))
        return ret

    # Check if the symlink to GCRON_TASKS is properly set in the shared storage
    if (not os.path.lexists(GCRON_UPDATE_TASK) or
        not os.path.lexists(GCRON_CROND_TASK) or
        os.readlink(GCRON_CROND_TASK) != GCRON_TASKS):
        print_str = ("Please run 'snap_scheduler.py' init to initialise "
                     "the snap scheduler for the local node.")
        log.error(print_str)
        output(print_str)
        ret = NODE_NOT_INITIALISED
        return ret

    # Check status of snapshot scheduler.
    if args.action == "status":
        ret = scheduler_status()
        if ret == 0:
            if scheduler_enabled:
                output("Snapshot scheduling status: Enabled")
            else:
                output("Snapshot scheduling status: Disabled")
        else:
            output("Failed to check status of snapshot scheduler")
        return ret

    # Enable snapshot scheduler
    if args.action == "enable":
        ret = enable_scheduler()
        if ret == 0:
            subprocess.Popen(["touch", "-h", GCRON_TASKS])
            gf_event (EVENT_SNAPSHOT_SCHEDULER_ENABLED,
                      status="Successfully Enabled")
        else:
            gf_event (EVENT_SNAPSHOT_SCHEDULER_ENABLE_FAILED,
                      error=print_error(ret))
        return ret

    # Disable snapshot scheduler
    if args.action == "disable":
        ret = disable_scheduler()
        if ret == 0:
            subprocess.Popen(["touch", "-h", GCRON_TASKS])
            gf_event (EVENT_SNAPSHOT_SCHEDULER_DISABLED,
                      status="Successfully Disabled")
        else:
            gf_event (EVENT_SNAPSHOT_SCHEDULER_DISABLE_FAILED,
                      error=print_error(ret))
        return ret

    # List snapshot schedules
    if args.action == "list":
        ret = list_schedules()
        return ret

    # Add snapshot schedules
    if args.action == "add":
        ret = syntax_checker(args)
        if ret != 0:
            return ret
        ret = add_schedules(args.jobname, args.schedule, args.volname)
        if ret == 0:
            subprocess.Popen(["touch", "-h", GCRON_TASKS])
            gf_event (EVENT_SNAPSHOT_SCHEDULER_SCHEDULE_ADDED,
                      status="Successfully added job "+args.jobname)
        else:
            gf_event (EVENT_SNAPSHOT_SCHEDULER_SCHEDULE_ADD_FAILED,
                      status="Failed to add job "+args.jobname,
                      error=print_error(ret))
        return ret

    # Delete snapshot schedules
    if args.action == "delete":
        ret = syntax_checker(args)
        if ret != 0:
            return ret
        ret = delete_schedules(args.jobname)
        if ret == 0:
            subprocess.Popen(["touch", "-h", GCRON_TASKS])
            gf_event (EVENT_SNAPSHOT_SCHEDULER_SCHEDULE_DELETED,
                      status="Successfully deleted job "+args.jobname)
        else:
            gf_event (EVENT_SNAPSHOT_SCHEDULER_SCHEDULE_DELETE_FAILED,
                      status="Failed to delete job "+args.jobname,
                      error=print_error(ret))
        return ret

    # Edit snapshot schedules
    if args.action == "edit":
        ret = syntax_checker(args)
        if ret != 0:
            return ret
        ret = edit_schedules(args.jobname, args.schedule, args.volname)
        if ret == 0:
            subprocess.Popen(["touch", "-h", GCRON_TASKS])
            gf_event (EVENT_SNAPSHOT_SCHEDULER_SCHEDULE_EDITED,
                      status="Successfully edited job "+args.jobname)
        else:
            gf_event (EVENT_SNAPSHOT_SCHEDULER_SCHEDULE_EDIT_FAILED,
                      status="Failed to edit job "+args.jobname,
                      error=print_error(ret))
        return ret

    ret = INVALID_ARG
    return ret

def gf_event(event_type, **kwargs):
    if EVENTS_ENABLED:
        from events.gf_event import gf_event as gfevent
        gfevent(event_type, **kwargs)


def main(argv):
    initLogger()
    ret = -1
    parser = argparse.ArgumentParser()
    subparsers = parser.add_subparsers(dest="action",
                                       metavar=('{init, status, enable,'
                                               ' disable, list, add,'
                                               ' delete, edit}'))
    subparsers.add_parser('init',
                          help="Initialise the node for snapshot scheduling")

    subparsers.add_parser("status",
                          help="Check if snapshot scheduling is "
                          "enabled or disabled")
    subparsers.add_parser("enable",
                          help="Enable snapshot scheduling")
    subparsers.add_parser("disable",
                          help="Disable snapshot scheduling")
    subparsers.add_parser("disable_force")
    subparsers.add_parser("list",
                          help="List snapshot schedules")
    parser_add = subparsers.add_parser("add",
                                       help="Add snapshot schedules")
    parser_add.add_argument("jobname", help="Job Name")
    parser_add.add_argument("schedule", help="Schedule")
    parser_add.add_argument("volname", help="Volume Name")

    parser_delete = subparsers.add_parser("delete",
                                          help="Delete snapshot schedules")
    parser_delete.add_argument("jobname", help="Job Name")
    parser_edit = subparsers.add_parser("edit",
                                        help="Edit snapshot schedules")
    parser_edit.add_argument("jobname", help="Job Name")
    parser_edit.add_argument("schedule", help="Schedule")
    parser_edit.add_argument("volname", help="Volume Name")

    args = parser.parse_args(argv)

    if not os.path.exists(SHARED_STORAGE_DIR):
        output("Failed: "+SHARED_STORAGE_DIR+" does not exist.")
        return SHARED_STORAGE_DIR_DOESNT_EXIST

    if not os.path.ismount(SHARED_STORAGE_DIR):
        output("Failed: Shared storage is not mounted at "+SHARED_STORAGE_DIR)
        return SHARED_STORAGE_NOT_MOUNTED

    if not os.path.exists(SHARED_STORAGE_DIR+"/snaps/"):
        try:
            os.makedirs(SHARED_STORAGE_DIR+"/snaps/")
        except OSError as e:
            if errno != EEXIST:
                log.error("Failed to create %s : %s", SHARED_STORAGE_DIR+"/snaps/", e)
                output("Failed to create %s. Error: %s"
                       % (SHARED_STORAGE_DIR+"/snaps/", e))
                return INTERNAL_ERROR

    if not os.path.exists(GCRON_ENABLED):
        f = os.open(GCRON_ENABLED, os.O_CREAT | os.O_NONBLOCK, 0o644)
        os.close(f)

    if not os.path.exists(LOCK_FILE_DIR):
        try:
            os.makedirs(LOCK_FILE_DIR)
        except OSError as e:
            if errno != EEXIST:
                log.error("Failed to create %s : %s", LOCK_FILE_DIR, e)
                output("Failed to create %s. Error: %s"
                       % (LOCK_FILE_DIR, e))
                return INTERNAL_ERROR

    try:
        f = os.open(LOCK_FILE, os.O_CREAT | os.O_RDWR | os.O_NONBLOCK, 0o644)
        try:
            fcntl.flock(f, fcntl.LOCK_EX | fcntl.LOCK_NB)
            ret = perform_operation(args)
            fcntl.flock(f, fcntl.LOCK_UN)
        except IOError:
            log.info("%s is being processed by another agent.", LOCK_FILE)
            output("Another snap_scheduler command is running. "
                   "Please try again after some time.")
            return ANOTHER_TRANSACTION_IN_PROGRESS
        os.close(f)
    except OSError as e:
        log.error("Failed to open %s : %s", LOCK_FILE, e)
        output("Failed to open %s. Error: %s" % (LOCK_FILE, e))
        return INTERNAL_ERROR

    return ret


if __name__ == "__main__":
    sys.exit(main(sys.argv[1:]))



#!/usr/bin/python3

import os
from optparse import OptionParser

if __name__ == '__main__':
    # check if swift is installed
    try:
        from gluster.swift.common.Glusterfs import get_mnt_point, unmount
    except ImportError:
        import sys
        sys.exit("Openstack Swift does not appear to be installed properly");

    op = OptionParser(usage="%prog [options...]")
    op.add_option('--volname', dest='vol', type=str)
    op.add_option('--last', dest='last', type=str)
    (opts, args) = op.parse_args()


    mnt_point = get_mnt_point(opts.vol)
    if mnt_point:
        unmount(mnt_point)
    else:
        sys.exit("get_mnt_point returned none for mount point")



#!/usr/bin/python3

from __future__ import print_function
import os
import sys

curdir = os.path.dirname(sys.argv[0])
gendir = os.path.join(curdir, '../../../../libglusterfs/src')
sys.path.append(gendir)
from generator import ops, fop_subs, cbk_subs, generate

FOPS_COMMON_TEMPLATE = """
int32_t
gf_utime_@NAME@ (call_frame_t *frame, xlator_t *this,
                @LONG_ARGS@)
{
        gl_timespec_get(&frame->root->ctime);

        (void) utime_update_attribute_flags(frame, this, GF_FOP_@UPNAME@);
        STACK_WIND (frame, gf_utime_@NAME@_cbk, FIRST_CHILD(this),
                    FIRST_CHILD(this)->fops->@NAME@, @SHORT_ARGS@);
        return 0;
}
"""

FOPS_CBK_COMMON_TEMPLATE = """
int32_t
gf_utime_@NAME@_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
                     int32_t op_ret, int32_t op_errno,
                    @LONG_ARGS@)
{
        STACK_UNWIND_STRICT (@NAME@, frame, op_ret, op_errno, @SHORT_ARGS@);
        return 0;
}
"""

FOPS_READ_TEMPLATE = """
int32_t
gf_utime_@NAME@ (call_frame_t *frame, xlator_t *this,
                @LONG_ARGS@)
{
        gl_timespec_get(&frame->root->ctime);

        (void) utime_update_attribute_flags(frame, this, GF_FOP_READ);
        STACK_WIND (frame, gf_utime_@NAME@_cbk, FIRST_CHILD(this),
                    FIRST_CHILD(this)->fops->@NAME@, @SHORT_ARGS@);
        return 0;
}
"""

FOPS_WRITE_TEMPLATE = """
int32_t
gf_utime_@NAME@ (call_frame_t *frame, xlator_t *this,
                @LONG_ARGS@)
{
        gl_timespec_get(&frame->root->ctime);

        (void) utime_update_attribute_flags(frame, this, GF_FOP_WRITE);
        STACK_WIND (frame, gf_utime_@NAME@_cbk, FIRST_CHILD(this),
                    FIRST_CHILD(this)->fops->@NAME@, @SHORT_ARGS@);
        return 0;
}
"""

FOPS_COPY_FILE_RANGE_TEMPLATE = """
int32_t
gf_utime_@NAME@ (call_frame_t *frame, xlator_t *this,
                @LONG_ARGS@)
{
        gl_timespec_get(&frame->root->ctime);

        (void) utime_update_attribute_flags(frame, this, GF_FOP_COPY_FILE_RANGE);
        STACK_WIND (frame, gf_utime_@NAME@_cbk, FIRST_CHILD(this),
                    FIRST_CHILD(this)->fops->@NAME@, @SHORT_ARGS@);
        return 0;
}
"""

FOPS_SETATTR_TEMPLATE = """
int32_t
gf_utime_@NAME@ (call_frame_t *frame, xlator_t *this,
             @LONG_ARGS@)
{
        gl_timespec_get(&frame->root->ctime);

        if (!valid) {
                frame->root->flags |= MDATA_CTIME;
        }

        if (valid & (GF_SET_ATTR_UID | GF_SET_ATTR_GID)) {
                frame->root->flags |= MDATA_CTIME;
        }

        if (valid & GF_SET_ATTR_MODE) {
                frame->root->flags |= MDATA_CTIME;
        }

        if (valid & (GF_SET_ATTR_ATIME | GF_SET_ATTR_MTIME)) {
            if (valid & GF_ATTR_ATIME_NOW) {
                frame->root->ctime.tv_sec = stbuf->ia_atime;
                frame->root->ctime.tv_nsec = stbuf->ia_atime_nsec;
            } else if (valid & GF_ATTR_MTIME_NOW) {
                frame->root->ctime.tv_sec = stbuf->ia_mtime;
                frame->root->ctime.tv_nsec = stbuf->ia_mtime_nsec;
            }
        }

        STACK_WIND (frame, gf_utime_@NAME@_cbk, FIRST_CHILD(this),
                    FIRST_CHILD(this)->fops->@NAME@, @SHORT_ARGS@);
        return 0;
}
"""

utime_ops = ['fallocate', 'zerofill', 'opendir', 'mknod', 'mkdir',
             'unlink', 'rmdir', 'symlink', 'rename', 'link', 'truncate',
             'ftruncate', 'create', 'open', 'removexattr', 'fremovexattr']

utime_read_op = ['readv']
utime_write_op = ['writev']
utime_setattr_ops = ['setattr', 'fsetattr']
utime_copy_file_range_ops = ['copy_file_range']

def gen_defaults():
    for name in ops:
        if name in utime_ops:
            print(generate(FOPS_CBK_COMMON_TEMPLATE, name, cbk_subs))
            print(generate(FOPS_COMMON_TEMPLATE, name, fop_subs))
        if name in utime_read_op:
            print(generate(FOPS_CBK_COMMON_TEMPLATE, name, cbk_subs))
            print(generate(FOPS_READ_TEMPLATE, name, fop_subs))
        if name in utime_write_op:
            print(generate(FOPS_CBK_COMMON_TEMPLATE, name, cbk_subs))
            print(generate(FOPS_WRITE_TEMPLATE, name, fop_subs))
        if name in utime_setattr_ops:
            print(generate(FOPS_CBK_COMMON_TEMPLATE, name, cbk_subs))
            print(generate(FOPS_SETATTR_TEMPLATE, name, fop_subs))
        if name in utime_copy_file_range_ops:
            print(generate(FOPS_CBK_COMMON_TEMPLATE, name, cbk_subs))
            print(generate(FOPS_COPY_FILE_RANGE_TEMPLATE, name, fop_subs))

for l in open(sys.argv[1], 'r').readlines():
    if l.find('#pragma generate') != -1:
        print("/* BEGIN GENERATED CODE - DO NOT MODIFY */")
        gen_defaults()
        print("/* END GENERATED CODE */")
    else:
        print(l[:-1])




from __future__ import print_function
import sys

try:
    import psutil
except ImportError:
    print("Please install psutil --> pip install psutil")
    sys.exit(1)

def pmap_find(p, name):
    for m in p.memory_maps(grouped=True):
        if m.path.endswith("%s.so" % name):
            return True
        continue
    return False

def pidof(processname):
    for p in psutil.process_iter():
        if p.pid == 0:
            continue
        if "gluster" in processname:
            if processname == "glusterd" and pmap_find(p, "glusterd"):
                print((p.pid))
            if processname == "glusterfs" and pmap_find(p, "client"):
                print((p.pid))
            if processname == "glusterfsd" and pmap_find(p, "posix-acl"):
                print((p.pid))
            continue
        if processname.strip() == p.name():
            print((p.pid))

def main(argv):
    if len(argv) < 2:
        sys.stderr.write("Usage: %s <processname>\n" % (argv[0],))
        return 1
    try:
        pidof(argv[1])
    except Exception as err:
        print(err)
        sys.stderr.write("Please be root - %s\n" % err);
        sys.exit(1)

if __name__ == "__main__":
    main(sys.argv)




from __future__ import print_function
import ctypes

api = ctypes.CDLL("libgfapi.so", mode=ctypes.RTLD_GLOBAL)

api.glfs_ipc.argtypes = [ ctypes.c_void_p, ctypes.c_int, ctypes.c_void_p, ctypes.c_void_p ]
api.glfs_ipc.restype = ctypes.c_int

def do_ipc (host, volume):
	fs = api.glfs_new(volume)
	#api.glfs_set_logging(fs, "/dev/stderr", 7)
	api.glfs_set_volfile_server(fs, "tcp", host, 24007)

	api.glfs_init(fs)
	ret  = api.glfs_ipc(fs, 1470369258, 0, 0)
	api.glfs_fini(fs)

	return ret

if __name__ == "__main__":
	import sys

	try:
		res = do_ipc(*sys.argv[1:3])
		print(res)
	except:
		print("IPC failed (volume not started?)")



# -*- coding: utf-8 -*-
from __future__ import print_function
from argparse import ArgumentParser, RawDescriptionHelpFormatter
import inspect
import subprocess
import os
import xml.etree.cElementTree as etree
import json
import sys

MY_UUID = None
parser = ArgumentParser(formatter_class=RawDescriptionHelpFormatter,
                        description=__doc__)
subparsers = parser.add_subparsers(dest="mode")

subcommands = {}
cache_data = {}
ParseError = etree.ParseError if hasattr(etree, 'ParseError') else SyntaxError
_common_args_func = lambda p: True


class GlusterCmdException(Exception):
    def __init__(self, message):
        self.message = message
        try:
            # Python 3
            super().__init__(message)
        except TypeError:
            # Python 2
            super(GlusterCmdException, self).__init__(message)


def get_node_uuid():
    # Caches the Node UUID in global variable,
    # Executes gluster system:: uuid get command only if
    # calling this function for first time
    global MY_UUID
    if MY_UUID is not None:
        return MY_UUID

    cmd = ["gluster", "system::", "uuid", "get", "--xml"]
    rc, out, err = execute(cmd)

    if rc != 0:
        return None

    tree = etree.fromstring(out)
    uuid_el = tree.find("uuidGenerate/uuid")
    MY_UUID = uuid_el.text
    return MY_UUID


def yesno(flag):
    return "Yes" if flag else "No"


def oknotok(flag):
    return "OK" if flag else "NOT OK"


def output_error(message, errcode=1):
    print (message, file=sys.stderr)
    sys.exit(errcode)


def node_output_ok(message=""):
    # Prints Success JSON output and exits with returncode zero
    out = {"ok": True, "nodeid": get_node_uuid(), "output": message}
    print (json.dumps(out))
    sys.exit(0)


def node_output_notok(message):
    # Prints Error JSON output and exits with returncode zero
    out = {"ok": False, "nodeid": get_node_uuid(), "error": message}
    print (json.dumps(out))
    sys.exit(0)


def execute(cmd):
    p = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE,
                         universal_newlines=True)
    out, err = p.communicate()
    return p.returncode, out, err


def get_pool_list():
    cmd = ["gluster", "--mode=script", "pool", "list", "--xml"]
    rc, out, err = execute(cmd)
    if rc != 0:
        output_error("Failed to get Pool Info: {0}".format(err))

    tree = etree.fromstring(out)

    pool = []
    try:
        for p in tree.findall('peerStatus/peer'):
            pool.append({"nodeid": p.find("uuid").text,
                         "hostname": p.find("hostname").text,
                         "connected": (True if p.find("connected").text == "1"
                                       else False)})
    except (ParseError, AttributeError, ValueError) as e:
        output_error("Failed to parse Pool Info: {0}".format(e))

    return pool


class NodeOutput(object):
    def __init__(self, **kwargs):
        self.nodeid = kwargs.get("nodeid", "")
        self.hostname = kwargs.get("hostname", "")
        self.node_up = kwargs.get("node_up", False)
        self.ok = kwargs.get("ok", False)
        self.output = kwargs.get("output", "N/A")
        self.error = kwargs.get("error", "N/A")


def execute_in_peers(name, args=[]):
    # Get the file name of Caller function, If the file name is peer_example.py
    # then Gluster peer command will be gluster system:: execute example.py
    # Command name is without peer_
    frame = inspect.stack()[1]
    module = inspect.getmodule(frame[0])
    actual_file = module.__file__
    # If file is symlink then find actual file
    if os.path.islink(actual_file):
        actual_file = os.readlink(actual_file)

    # Get the name of file without peer_
    cmd_name = os.path.basename(actual_file).replace("peer_", "")
    cmd = ["gluster", "system::", "execute", cmd_name, name] + args
    rc, out, err = execute(cmd)
    if rc != 0:
        raise GlusterCmdException((rc, out, err, " ".join(cmd)))

    out = out.strip().splitlines()

    # JSON decode each line and construct one object with node id as key
    all_nodes_data = {}
    for node_data in out:
        data = json.loads(node_data)
        all_nodes_data[data["nodeid"]] = {
            "nodeid": data.get("nodeid"),
            "ok": data.get("ok"),
            "output": data.get("output", ""),
            "error": data.get("error", "")}

    # gluster pool list
    pool_list = get_pool_list()

    data_out = []
    # Iterate pool_list and merge all_nodes_data collected above
    # If a peer node is down then set node_up = False
    for p in pool_list:
        p_data = all_nodes_data.get(p.get("nodeid"), None)
        row_data = NodeOutput(node_up=False,
                              hostname=p.get("hostname"),
                              nodeid=p.get("nodeid"),
                              ok=False)

        if p_data is not None:
            # Node is UP
            row_data.node_up = True
            row_data.ok = p_data.get("ok")
            row_data.output = p_data.get("output")
            row_data.error = p_data.get("error")

        data_out.append(row_data)

    return data_out


def sync_file_to_peers(fname):
    # Copy file from current node to all peer nodes, fname
    # is path after GLUSTERD_WORKDIR
    cmd = ["gluster", "system::", "copy", "file", fname]
    rc, out, err = execute(cmd)
    if rc != 0:
        raise GlusterCmdException((rc, out, err))


class Cmd(object):
    name = ""

    def run(self, args):
        # Must required method. Raise NotImplementedError if derived class
        # not implemented this method
        raise NotImplementedError("\"run(self, args)\" method is "
                                  "not implemented by \"{0}\"".format(
                                      self.__class__.__name__))


def runcli():
    # Get list of Classes derived from class "Cmd" and create
    # a subcommand as specified in the Class name. Call the args
    # method by passing subcommand parser, Derived class can add
    # arguments to the subcommand parser.
    metavar_data = []
    for c in Cmd.__subclasses__():
        cls = c()
        if getattr(cls, "name", "") == "":
            raise NotImplementedError("\"name\" is not added "
                                      "to \"{0}\"".format(
                                          cls.__class__.__name__))

        # Do not show in help message if subcommand starts with node-
        if not cls.name.startswith("node-"):
            metavar_data.append(cls.name)

        p = subparsers.add_parser(cls.name)
        args_func = getattr(cls, "args", None)
        if args_func is not None:
            args_func(p)

        # Apply common args if any
        _common_args_func(p)

        # A dict to save subcommands, key is name of the subcommand
        subcommands[cls.name] = cls

    # Hide node commands in Help message
    subparsers.metavar = "{" + ",".join(metavar_data) + "}"

    # Get all parsed arguments
    args = parser.parse_args()

    # Get the subcommand to execute
    cls = subcommands.get(args.mode, None)

    # Run
    if cls is not None:
        cls.run(args)


def set_common_args_func(func):
    global _common_args_func
    _common_args_func = func



# -*- coding: utf-8 -*-

# Copyright (c) 2015 Red Hat, Inc. <http://www.redhat.com/>
# This file is part of GlusterFS.
#
# This file is licensed to you under your choice of the GNU Lesser
# General Public License, version 3 or any later version (LGPLv3 or
# later), or the GNU General Public License, version 2 (GPLv2), in all
# cases as published by the Free Software Foundation.

import os
from ctypes import CDLL, RTLD_GLOBAL, get_errno, create_string_buffer, c_ulong, byref
from ctypes.util import find_library
from gfind_py2py3 import bytearray_to_str, gf_create_string_buffer
from gfind_py2py3 import gfind_history_changelog, gfind_changelog_register
from gfind_py2py3 import gfind_history_changelog_done


class ChangelogException(OSError):
    pass

libgfc = CDLL(find_library("gfchangelog"), mode=RTLD_GLOBAL, use_errno=True)


def raise_oserr(prefix=None):
    errn = get_errno()
    prefix_or_empty = prefix + ": " if prefix else ""
    raise ChangelogException(errn, prefix_or_empty + os.strerror(errn))


def cl_init():
    ret = libgfc.gf_changelog_init(None)
    if ret == -1:
        raise_oserr(prefix="gf_changelog_init")


def cl_register(brick, path, log_file, log_level, retries=0):
    ret = gfind_changelog_register(libgfc, brick, path, log_file,log_level, retries)
    if ret == -1:
        raise_oserr(prefix="gf_changelog_register")


def cl_history_scan():
    ret = libgfc.gf_history_changelog_scan()
    if ret == -1:
        raise_oserr(prefix="gf_history_changelog_scan")

    return ret


def cl_history_changelog(changelog_path, start, end, num_parallel):
    actual_end = c_ulong()
    ret = gfind_history_changelog(libgfc,changelog_path, start, end,
                                      num_parallel,
                                      byref(actual_end))
    if ret == -1:
        raise_oserr(prefix="gf_history_changelog")

    return actual_end.value


def cl_history_startfresh():
    ret = libgfc.gf_history_changelog_start_fresh()
    if ret == -1:
        raise_oserr(prefix="gf_history_changelog_start_fresh")


def cl_history_getchanges():
    """ remove hardcoding for path name length """
    def clsort(f):
        return f.split('.')[-1]

    changes = []
    buf = gf_create_string_buffer(4096)

    while True:
        ret = libgfc.gf_history_changelog_next_change(buf, 4096)
        if ret in (0, -1):
            break
        # py2 and py3 compatibility
        result = bytearray_to_str(buf.raw[:ret - 1])
        changes.append(result)
    if ret == -1:
        raise_oserr(prefix="gf_history_changelog_next_change")

    return sorted(changes, key=clsort)


def cl_history_done(clfile):
    ret = gfind_history_changelog_done(libgfc, clfile)
    if ret == -1:
        raise_oserr(prefix="gf_history_changelog_done")



#!/usr/bin/python3

import string

# ops format: 'fop-arg' name type stub-field [nosync]
#             'cbk-arg' name type
#             'extra'   name type arg-str
#             'journal' fop-type
#             'link'    inode iatt
#
# 'role' indicates the significance of this line to the code generator (sort of
# our own type).
#
# For fop-arg, we first need to know the name and the type of the arg so that
# we can generate SHORT_ARGS (for function calls) and LONG_ARGS (for
# declarations).  For code that uses stubs, we also need to know the name of
# the stub field, which might be different than the argument itself.  Lastly,
# for code that uses syncops, we need to know whether whoever wrote the syncop
# for this fop "forgot" to include this argument.  (Editorial: this kind of
# creeping inconsistency is why we should have used code generation for stubs
# and syncops as well as defaults all along.)  To address this need, we use the
# optional 'nosync' field for arguments (e.g. mkdir.umask) that we should skip
# in generated syncop code.
#
# 'cbk-arg' is like fop-arg but simpler and used for generating callbacks
# instead of fop functions.
#
# 'extra' is also like fop-arg, but it's another hack for syncops.  This time
# the problem is that some of what would normally be *callback* arguments are
# instead created in the caller and passed to the syncop.  We handle that by
# adding an entry at the appropriate place in the fop-arg list, with the name
# and type to generate a declaration and an argument string to generate the
# actual syncop call.
#
# The mere presence of a 'journal' item is sufficient for most of the journal
# code to recognize that it should do something.  However, reconciliation also
# needs to decide how reconciliation builds the arguments it needs to call down
# to the syncop layer, based on what's in the journal.  To do that, we divide
# ops into three types and store those types in the ops table.  In general,
# these three types work as follows.
#
#    For an fd-op, the GFID in the journal is used (in loc.gfid) field to
#    look up an inode, then an anonymous fd is found/created for that inode.
#
#    For an inode-op, the GFID in the journal is used the same way, but no fd
#    is needed.
#
#    For an entry-op, the *parent* GFID and name from the journal are used to
#    look up an inode (via loc.pargfid and par.name respectively).
#
# The only places this seems to fall down is for link and create.  In link,
# which is generally an entry-op, the source is looked up as though it's an
# inode-op.  In create, we have an fd argument but it's really a return
# argument so we get a fresh inode instead of looking one up.  Those two cases
# need to be handled as special cases in the reconciliation code.
#
# 'link' is (hopefully) the last of the journal/syncop hacks.  Much like
# 'extra', some values that are returned as callback arguments in the normal
# case are handled differently for syncops.  For syncops that create objects
# (e.g. mkdir) we need to link those objects into our inode table.  The 'inode'
# and 'iatt' fields here give us the information we need to construct the
# proper inode_link call(s).

ops = {}
xlator_cbks = {}
xlator_dumpops = {}

ops['fgetxattr'] = (
	('fop-arg', 'fd',			'fd_t *'),
	('fop-arg',	'name',			'const char *'),
	('fop-arg',	'xdata',		'dict_t *'),
	('cbk-arg',	'dict',			'dict_t *'),
	('cbk-arg',	'xdata',		'dict_t *'),
)

ops['fsetxattr'] = (
	('fop-arg',	'fd',			'fd_t *',			'fd'),
	('fop-arg',	'dict',			'dict_t *',			'xattr'),
	('fop-arg',	'flags',		'int32_t',			'flags'),
	('fop-arg',	'xdata',		'dict_t *',			'xdata'),
	('cbk-arg',	'xdata',		'dict_t *'),
	('journal',	'fd-op'),
)

ops['setxattr'] = (
	('fop-arg',	'loc',			'loc_t *',			'loc'),
	('fop-arg',	'dict',			'dict_t *',			'xattr'),
	('fop-arg',	'flags',		'int32_t',			'flags'),
	('fop-arg',	'xdata',		'dict_t *',			'xdata'),
	('cbk-arg',	'xdata',		'dict_t *'),
	('journal',	'inode-op'),
)

ops['statfs'] = (
	('fop-arg',	'loc',			'loc_t *'),
	('fop-arg',	'xdata',		'dict_t *'),
	('cbk-arg',	'buf',			'struct statvfs *'),
	('cbk-arg',	'xdata',		'dict_t *'),
)

ops['fsyncdir'] = (
	('fop-arg',	'fd',			'fd_t *'),
	('fop-arg',	'flags',		'int32_t'),
	('fop-arg',	'xdata',		'dict_t *'),
	('cbk-arg',	'xdata',		'dict_t *'),
)

ops['opendir'] = (
	('fop-arg',	'loc',			'loc_t *'),
	('fop-arg',	'fd',			'fd_t *'),
	('fop-arg',	'xdata',		'dict_t *'),
	('cbk-arg',	'fd',			'fd_t *'),
	('cbk-arg',	'xdata',		'dict_t *'),
)

ops['fstat'] = (
	('fop-arg',	'fd',			'fd_t *'),
	('fop-arg',	'xdata',		'dict_t *'),
	('cbk-arg',	'buf',			'struct iatt *'),
	('cbk-arg',	'xdata',		'dict_t *'),
)

ops['fsync'] = (
	('fop-arg',	'fd',			'fd_t *'),
	('fop-arg',	'flags',		'int32_t'),
	('extra',	'preop',		'struct iatt',		'&preop'),
	('extra',	'postop',		'struct iatt',		'&postop'),
	('fop-arg',	'xdata',		'dict_t *'),
	('cbk-arg',	'prebuf',		'struct iatt *'),
	('cbk-arg',	'postbuf',		'struct iatt *'),
	('cbk-arg',	'xdata',		'dict_t *'),
)

ops['flush'] = (
	('fop-arg',	'fd',			'fd_t *'),
	('fop-arg',	'xdata',		'dict_t *'),
	('cbk-arg',	'xdata',		'dict_t *'),
)

ops['writev'] = (
	('fop-arg',	'fd',			'fd_t *',			'fd'),
	('fop-arg',	'vector',		'struct iovec *',	'vector'),
	('fop-arg',	'count',		'int32_t'),
	('fop-arg',	'off',			'off_t',			'offset'),
	('fop-arg',	'flags',		'uint32_t',			'flags'),
	('fop-arg',	'iobref',		'struct iobref *'),
	('extra',	'preop',		'struct iatt',		'&preop'),
	('extra',	'postop',		'struct iatt',		'&postop'),
	('fop-arg',	'xdata',		'dict_t *',			'xdata'),
	('cbk-arg',	'prebuf',		'struct iatt *'),
	('cbk-arg',	'postbuf',		'struct iatt *'),
	('cbk-arg',	'xdata',		'dict_t *'),
	('journal',	'fd-op'),
)

ops['readv'] = (
	('fop-arg',	'fd',			'fd_t *'),
	('fop-arg',	'size',			'size_t'),
	('fop-arg',	'offset',		'off_t'),
	('fop-arg',	'flags',		'uint32_t'),
	('extra',	'iatt',			'struct iatt',		'&iatt'),
	('fop-arg',	'xdata',		'dict_t *'),
	('cbk-arg',	'vector',		'struct iovec *'),
	('cbk-arg',	'count',		'int32_t'),
	('cbk-arg',	'stbuf',		'struct iatt *'),
	('cbk-arg',	'iobref',		'struct iobref *'),
	('cbk-arg',	'xdata',		'dict_t *'),
)

ops['open'] = (
	('fop-arg',	'loc',			'loc_t *'),
	('fop-arg',	'flags',		'int32_t'),
	('fop-arg',	'fd',			'fd_t *'),
	('fop-arg',	'xdata',		'dict_t *'),
	('cbk-arg',	'fd',			'fd_t *'),
	('cbk-arg',	'xdata',		'dict_t *'),
)

ops['create'] = (
	('fop-arg',	'loc',			'loc_t *',			'loc'),
	('fop-arg',	'flags',		'int32_t',			'flags'),
	('fop-arg',	'mode',			'mode_t',			'mode'),
	('fop-arg',	'umask',		'mode_t',			'umask',	'nosync'),
	('fop-arg',	'fd',			'fd_t *',			'fd'),
	('extra',	'iatt',			'struct iatt',		'&iatt'),
	('fop-arg',	'xdata',		'dict_t *',			'xdata'),
	('cbk-arg',	'fd',			'fd_t *'),
	('cbk-arg',	'inode',		'inode_t *'),
	('cbk-arg',	'buf',			'struct iatt *'),
	('cbk-arg',	'preparent',	'struct iatt *'),
	('cbk-arg',	'postparent',	'struct iatt *'),
	('cbk-arg',	'xdata',		'dict_t *'),
	('journal',	'entry-op'),
	('link',	'loc.inode',	'&iatt'),
)

ops['link'] = (
	('fop-arg',	'oldloc',		'loc_t *',			'loc'),
	('fop-arg',	'newloc',		'loc_t *',			'loc2'),
	('extra',	'iatt',			'struct iatt',		'&iatt'),
	('fop-arg',	'xdata',		'dict_t *',			'xdata'),
	('cbk-arg',	'inode',		'inode_t *'),
	('cbk-arg',	'buf',			'struct iatt *'),
	('cbk-arg',	'preparent',	'struct iatt *'),
	('cbk-arg',	'postparent',	'struct iatt *'),
	('cbk-arg',	'xdata',		'dict_t *'),
	('journal',	'entry-op'),
)

ops['rename'] = (
	('fop-arg',	'oldloc',		'loc_t *',			'loc'),
	('fop-arg',	'newloc',		'loc_t *',			'loc2'),
	('fop-arg',	'xdata',		'dict_t *',			'xdata'),
	('cbk-arg',	'buf',			'struct iatt *'),
	('cbk-arg',	'preoldparent',	'struct iatt *'),
	('cbk-arg',	'postoldparent', 'struct iatt *'),
	('cbk-arg',	'prenewparent',	'struct iatt *'),
	('cbk-arg',	'postnewparent', 'struct iatt *'),
	('cbk-arg',	'xdata',		'dict_t *'),
	('journal',	'entry-op'),
)

ops['symlink'] = (
	('fop-arg',	'linkpath',		'const char *',		'linkname'),
	('fop-arg',	'loc',			'loc_t *',			'loc'),
	('fop-arg',	'umask',		'mode_t',			'mode',		'nosync'),
	('extra',	'iatt',			'struct iatt',		'&iatt'),
	('fop-arg',	'xdata',		'dict_t *',			'xdata'),
	('cbk-arg',	'inode',		'inode_t *'),
	('cbk-arg',	'buf',			'struct iatt *'),
	('cbk-arg',	'preparent',	'struct iatt *'),
	('cbk-arg',	'postparent',	'struct iatt *'),
	('cbk-arg',	'xdata',		'dict_t *'),
	('journal',	'entry-op'),
)

ops['rmdir'] = (
	('fop-arg',	'loc',			'loc_t *',			'loc'),
	('fop-arg',	'flags',		'int32_t',			'flags'),
	('fop-arg',	'xdata',		'dict_t *',			'xdata'),
	('cbk-arg',	'preparent',	'struct iatt *'),
	('cbk-arg',	'postparent',	'struct iatt *'),
	('cbk-arg',	'xdata',		'dict_t *'),
	('journal',	'entry-op'),
)

ops['unlink'] = (
	('fop-arg',	'loc',			'loc_t *',			'loc'),
	('fop-arg',	'flags',		'int32_t',			'flags',	'nosync'),
	('fop-arg',	'xdata',		'dict_t *',			'xdata'),
	('cbk-arg',	'preparent',	'struct iatt *'),
	('cbk-arg',	'postparent',	'struct iatt *'),
	('cbk-arg',	'xdata',		'dict_t *'),
	('journal',	'entry-op'),
)

ops['mkdir'] = (
	('fop-arg',	'loc',			'loc_t *',			'loc'),
	('fop-arg',	'mode',			'mode_t',			'mode'),
	('fop-arg',	'umask',		'mode_t',			'umask',	'nosync'),
	('extra',	'iatt',			'struct iatt',		'&iatt'),
	('fop-arg',	'xdata',		'dict_t *',			'xdata'),
	('cbk-arg',	'inode',		'inode_t *'),
	('cbk-arg',	'buf',			'struct iatt *'),
	('cbk-arg',	'preparent',	'struct iatt *'),
	('cbk-arg',	'postparent',	'struct iatt *'),
	('cbk-arg',	'xdata',		'dict_t *'),
	('journal',	'entry-op'),
	('link',	'loc.inode',	'&iatt'),
)

ops['mknod'] = (
	('fop-arg',	'loc',			'loc_t *',			'loc'),
	('fop-arg',	'mode',			'mode_t',			'mode'),
	('fop-arg',	'rdev',			'dev_t',			'rdev'),
	('fop-arg',	'umask',		'mode_t',			'umask',	'nosync'),
	('extra',	'iatt',			'struct iatt',		'&iatt'),
	('fop-arg',	'xdata',		'dict_t *',			'xdata'),
	('cbk-arg',	'inode',		'inode_t *'),
	('cbk-arg',	'buf',			'struct iatt *'),
	('cbk-arg',	'preparent',	'struct iatt *'),
	('cbk-arg',	'postparent',	'struct iatt *'),
	('cbk-arg',	'xdata',		'dict_t *'),
	('journal',	'entry-op'),
)

ops['readlink'] = (
	('fop-arg',	'loc',			'loc_t *'),
	('fop-arg',	'size',			'size_t'),
	('fop-arg',	'xdata',		'dict_t *'),
	('cbk-arg',	'path',			'const char *'),
	('cbk-arg',	'buf',			'struct iatt *'),
	('cbk-arg',	'xdata',		'dict_t *'),
)

ops['access'] = (
	('fop-arg',	'loc',			'loc_t *'),
	('fop-arg',	'mask',			'int32_t'),
	('fop-arg',	'xdata',		'dict_t *'),
	('cbk-arg',	'xdata',		'dict_t *'),
)

ops['ftruncate'] = (
	('fop-arg',	'fd',			'fd_t *',				'fd'),
	('fop-arg',	'offset',		'off_t',				'offset'),
	('extra',	'preop',		'struct iatt',		'&preop'),
	('extra',	'postop',		'struct iatt',		'&postop'),
	('fop-arg',	'xdata',		'dict_t *',				'xdata'),
	('cbk-arg',	'prebuf',		'struct iatt *'),
	('cbk-arg',	'postbuf',		'struct iatt *'),
	('cbk-arg',	'xdata',		'dict_t *'),
	('journal',	'fd-op'),
)

ops['getxattr'] = (
	('fop-arg',	'loc',			'loc_t *'),
	('fop-arg',	'name',			'const char *'),
	('fop-arg',	'xdata',		'dict_t *'),
	('cbk-arg',	'dict',			'dict_t *'),
	('cbk-arg',	'xdata',		'dict_t *'),
)

ops['xattrop'] = (
	('fop-arg',	'loc',			'loc_t *',				'loc'),
	('fop-arg',	'flags',		'gf_xattrop_flags_t',	'optype'),
	('fop-arg',	'dict',			'dict_t *',				'xattr'),
	('fop-arg',	'xdata',		'dict_t *',				'xdata'),
	('cbk-arg',	'dict',			'dict_t *'),
	('cbk-arg',	'xdata',		'dict_t *'),
	('journal',	'inode-op'),
)

ops['fxattrop'] = (
	('fop-arg',	'fd',			'fd_t *',				'fd'),
	('fop-arg',	'flags',		'gf_xattrop_flags_t',	'optype'),
	('fop-arg',	'dict',			'dict_t *',				'xattr'),
	('fop-arg',	'xdata',		'dict_t *',				'xdata'),
	('cbk-arg',	'dict',			'dict_t *'),
	('cbk-arg',	'xdata',		'dict_t *'),
	('journal',	'fd-op'),
)

ops['removexattr'] = (
	('fop-arg',	'loc',			'loc_t *',			'loc'),
	('fop-arg',	'name',			'const char *',		'name'),
	('fop-arg',	'xdata',		'dict_t *',			'xdata'),
	('cbk-arg',	'xdata',		'dict_t *'),
	('journal',	'inode-op'),
)

ops['fremovexattr'] = (
	('fop-arg',	'fd',			'fd_t *',			'fd'),
	('fop-arg',	'name',			'const char *',		'name'),
	('fop-arg',	'xdata',		'dict_t *',			'xdata'),
	('cbk-arg',	'xdata',		'dict_t *'),
	('journal',	'fd-op'),
)

ops['lk'] = (
	('fop-arg',	'fd',			'fd_t *'),
	('fop-arg',	'cmd',			'int32_t'),
	('fop-arg',	'lock',			'struct gf_flock *'),
	('fop-arg',	'xdata',		'dict_t *'),
	('cbk-arg',	'lock',			'struct gf_flock *'),
	('cbk-arg',	'xdata',		'dict_t *'),
)

ops['inodelk'] = (
	('fop-arg',	'volume',		'const char *'),
	('fop-arg',	'loc',			'loc_t *'),
	('fop-arg',	'cmd',			'int32_t'),
	('fop-arg',	'lock',			'struct gf_flock *'),
	('fop-arg',	'xdata',		'dict_t *'),
	('cbk-arg',	'xdata',		'dict_t *'),
)

ops['finodelk'] = (
	('fop-arg',	'volume',		'const char *'),
	('fop-arg',	'fd',			'fd_t *'),
	('fop-arg',	'cmd',			'int32_t'),
	('fop-arg',	'lock',			'struct gf_flock *'),
	('fop-arg',	'xdata',		'dict_t *'),
	('cbk-arg',	'xdata',		'dict_t *'),
)

ops['entrylk'] = (
	('fop-arg',	'volume',		'const char *'),
	('fop-arg',	'loc',			'loc_t *'),
	('fop-arg',	'basename',		'const char *'),
	('fop-arg',	'cmd',			'entrylk_cmd'),
	('fop-arg',	'type',			'entrylk_type'),
	('fop-arg',	'xdata',		'dict_t *'),
	('cbk-arg',	'xdata',		'dict_t *'),
)

ops['fentrylk'] = (
	('fop-arg',	'volume',		'const char *'),
	('fop-arg',	'fd',			'fd_t *'),
	('fop-arg',	'basename',		'const char *'),
	('fop-arg',	'cmd',			'entrylk_cmd'),
	('fop-arg',	'type',			'entrylk_type'),
	('fop-arg',	'xdata',		'dict_t *'),
	('cbk-arg',	'xdata',		'dict_t *'),
)

ops['rchecksum'] = (
	('fop-arg',	'fd',			'fd_t *'),
	('fop-arg',	'offset',		'off_t'),
	('fop-arg',	'len',			'int32_t'),
	('fop-arg',	'xdata',		'dict_t *'),
	('cbk-arg',	'weak_cksum',	'uint32_t'),
	('cbk-arg',	'strong_cksum',	'uint8_t *'),
	('cbk-arg',	'xdata',		'dict_t *'),
)

ops['readdir'] = (
	('fop-arg',	'fd',			'fd_t *'),
	('fop-arg',	'size',			'size_t'),
	('fop-arg',	'off',			'off_t'),
	('fop-arg',	'xdata',		'dict_t *'),
	('cbk-arg',	'entries',		'gf_dirent_t *'),
	('cbk-arg',	'xdata',		'dict_t *'),
)

ops['readdirp'] = (
	('fop-arg',	'fd',			'fd_t *'),
	('fop-arg',	'size',			'size_t'),
	('fop-arg',	'off',			'off_t'),
	('fop-arg',	'xdata',		'dict_t *'),
	('cbk-arg',	'entries',		'gf_dirent_t *'),
	('cbk-arg',	'xdata',		'dict_t *'),
)

ops['setattr'] = (
	('fop-arg',	'loc',			'loc_t *',			'loc'),
	('fop-arg',	'stbuf',		'struct iatt *',	'stat'),
	('fop-arg',	'valid',		'int32_t',			'valid'),
	('extra',	'preop',		'struct iatt',		'&preop'),
	('extra',	'postop',		'struct iatt',		'&postop'),
	('fop-arg',	'xdata',		'dict_t *',			'xdata'),
	('cbk-arg',	'statpre',		'struct iatt *'),
	('cbk-arg',	'statpost',		'struct iatt *'),
	('cbk-arg',	'xdata',		'dict_t *'),
	('journal',	'inode-op'),
)

ops['truncate'] = (
	('fop-arg',	'loc',			'loc_t *',			'loc'),
	('fop-arg',	'offset',		'off_t',			'offset'),
	('fop-arg',	'xdata',		'dict_t *',			'xdata'),
	('cbk-arg',	'prebuf',		'struct iatt *'),
	('cbk-arg',	'postbuf',		'struct iatt *'),
	('cbk-arg',	'xdata',		'dict_t *'),
	('journal',	'inode-op'),
)

ops['stat'] = (
	('fop-arg',	'loc',			'loc_t *'),
	('fop-arg',	'xdata',		'dict_t *'),
	('cbk-arg',	'buf',			'struct iatt *'),
	('cbk-arg',	'xdata',		'dict_t *'),
)

ops['lookup'] = (
	('fop-arg',	'loc',			'loc_t *'),
	('fop-arg',	'xdata',		'dict_t *'),
	('cbk-arg',	'inode',		'inode_t *'),
	('cbk-arg',	'buf',			'struct iatt *'),
	('cbk-arg',	'xdata',		'dict_t *'),
	# We could add xdata everywhere automatically if somebody hadn't put
	# something after it here.
	('cbk-arg',	'postparent',	'struct iatt *'),
)

ops['fsetattr'] = (
	('fop-arg',	'fd',			'fd_t *',			'fd'),
	('fop-arg',	'stbuf',		'struct iatt *',	'stat'),
	('fop-arg',	'valid',		'int32_t',			'valid'),
	('extra',	'preop',		'struct iatt',		'&preop'),
	('extra',	'postop',		'struct iatt',		'&postop'),
	('fop-arg',	'xdata',		'dict_t *',			'xdata'),
	('cbk-arg',	'statpre',		'struct iatt *'),
	('cbk-arg',	'statpost',		'struct iatt *'),
	('cbk-arg',	'xdata',		'dict_t *'),
	('journal',	'fd-op'),
)

ops['fallocate'] = (
	('fop-arg',	'fd',			'fd_t *',			'fd'),
	('fop-arg',	'keep_size',	'int32_t',			'mode'),
	('fop-arg',	'offset',		'off_t',			'offset'),
	('fop-arg',	'len',			'size_t',			'size'),
	('fop-arg',	'xdata',		'dict_t *',			'xdata'),
	('cbk-arg',	'pre',			'struct iatt *'),
	('cbk-arg',	'post',			'struct iatt *'),
	('cbk-arg',	'xdata',		'dict_t *'),
	('journal',	'fd-op'),
)

ops['discard'] = (
	('fop-arg',	'fd',			'fd_t *',			'fd'),
	('fop-arg',	'offset',		'off_t',			'offset'),
	('fop-arg',	'len',			'size_t',			'size'),
	('fop-arg',	'xdata',		'dict_t *',			'xdata'),
	('cbk-arg',	'pre',			'struct iatt *'),
	('cbk-arg',	'post',			'struct iatt *'),
	('cbk-arg',	'xdata',		'dict_t *'),
	('journal',	'fd-op'),
)

ops['zerofill'] = (
	('fop-arg',	'fd',			'fd_t *',			'fd'),
	('fop-arg',	'offset',		'off_t',			'offset'),
	# As e.g. fallocate/discard (above) "len" should really be a size_t.
	('fop-arg',	'len',			'off_t',			'size'),
	('fop-arg',	'xdata',		'dict_t *',			'xdata'),
	('cbk-arg',	'pre',			'struct iatt *'),
	('cbk-arg',	'post',			'struct iatt *'),
	('cbk-arg',	'xdata',		'dict_t *'),
	('journal',	'fd-op'),
)

ops['ipc'] = (
	('fop-arg',	'op',			'int32_t'),
	('fop-arg',	'xdata',		'dict_t *',			'xdata'),
	('cbk-arg',	'xdata',		'dict_t *'),
	('journal',	'fd-op'),
)

ops['seek'] = (
	('fop-arg',	'fd',			'fd_t *'),
	('fop-arg',	'offset',		'off_t'),
	('fop-arg',	'what',			'gf_seek_what_t'),
	('fop-arg',	'xdata',		'dict_t *'),
	('cbk-arg',	'offset',		'off_t'),
	('cbk-arg',	'xdata',		'dict_t *'),
)

ops['getspec'] = (
	('fop-arg',	'key',			'const char *'),
	('fop-arg',	'flags',		'int32_t'),
	('cbk-arg',	'spec_data',	'char *'),
)

ops['lease'] = (
	('fop-arg',     'loc',                  'loc_t *'),
	('fop-arg',     'lease',                'struct gf_lease *'),
	('fop-arg',     'xdata',                'dict_t *'),
	('cbk-arg',     'lease',                'struct gf_lease *'),
	('cbk-arg',     'xdata',                'dict_t *'),
)

ops['getactivelk'] = (
	('fop-arg',     'loc',                  'loc_t *'),
	('fop-arg',     'xdata',                'dict_t *'),
	('cbk-arg',     'locklist',             'lock_migration_info_t *'),
	('cbk-arg',     'xdata',                'dict_t *'),
)

ops['setactivelk'] = (
	('fop-arg',     'loc',                  'loc_t *'),
	('fop-arg',     'locklist',             'lock_migration_info_t *'),
	('fop-arg',     'xdata',                'dict_t *'),
	('cbk-arg',     'xdata',                'dict_t *'),
)

ops['put'] = (
	('fop-arg',     'loc',                  'loc_t *',                      'loc'),
	('fop-arg',     'mode',                 'mode_t',                       'mode'),
	('fop-arg',     'umask',                'mode_t',                       'umask'),
	('fop-arg',     'flags',                'uint32_t',                     'flags'),
	('fop-arg',     'vector',               'struct iovec *',               'vector'),
	('fop-arg',     'count',                'int32_t'),
	('fop-arg',     'off',                  'off_t',                        'offset'),
	('fop-arg',     'iobref',               'struct iobref *'),
	('fop-arg',     'dict',                 'dict_t *',                     'xattr'),
	('fop-arg',     'xdata',                'dict_t *',                     'xdata'),
	('cbk-arg',     'inode',                'inode_t *'),
	('cbk-arg',     'buf',                  'struct iatt *'),
	('cbk-arg',     'preparent',            'struct iatt *'),
	('cbk-arg',     'postparent',           'struct iatt *'),
	('cbk-arg',     'xdata',                'dict_t *'),
)

ops['icreate'] = (
	('fop-arg',     'loc',                   'loc_t *'),
	('fop-arg',     'mode',                  'mode_t'),
	('fop-arg',     'xdata',                 'dict_t *'),
	('cbk-arg',     'inode',                 'inode_t *'),
	('cbk-arg',     'buf',                   'struct iatt *'),
	('cbk-arg',     'xdata',                 'dict_t *'),
)

ops['namelink'] = (
	('fop-arg',     'loc',                   'loc_t *'),
	('fop-arg',     'xdata',                 'dict_t *'),
	('cbk-arg',     'prebuf',                'struct iatt *'),
	('cbk-arg',     'postbuf',               'struct iatt *'),
	('cbk-arg',     'xdata',                 'dict_t *'),
)

ops['copy_file_range'] = (
        ('fop-arg',     'fd_in',                 'fd_t *'),
        ('fop-arg',     'off_in',                'off64_t '),
        ('fop-arg',     'fd_out',                'fd_t *'),
        ('fop-arg',     'off_out',               'off64_t '),
        ('fop-arg',     'len',                   'size_t'),
        ('fop-arg',     'flags',                 'uint32_t'),
        ('fop-arg',     'xdata',                 'dict_t *'),
        ('cbk-arg',     'stbuf',                 'struct iatt *'),
        ('cbk-arg',     'prebuf_dst',            'struct iatt *'),
        ('cbk-arg',     'postbuf_dst',           'struct iatt *'),
        ('cbk-arg',     'xdata',                 'dict_t *'),
)
#####################################################################
xlator_cbks['forget'] = (
	('fn-arg',      'this',        'xlator_t *'),
	('fn-arg',      'inode',       'inode_t *'),
	('ret-val',     'int32_t',     '0'),
)

xlator_cbks['release'] = (
	('fn-arg',      'this',        'xlator_t *'),
	('fn-arg',      'fd',          'fd_t *'),
	('ret-val',     'int32_t',     '0'),
)

xlator_cbks['releasedir'] = (
	('fn-arg',      'this',        'xlator_t *'),
	('fn-arg',      'fd',          'fd_t *'),
	('ret-val',     'int32_t',     '0'),
)

xlator_cbks['invalidate'] = (
	('fn-arg',      'this',        'xlator_t *'),
	('fn-arg',      'inode',       'inode_t *'),
	('ret-val',     'int32_t',     '0'),
)

xlator_cbks['client_destroy'] = (
	('fn-arg',      'this',        'xlator_t *'),
	('fn-arg',      'client',      'client_t *'),
	('ret-val',     'int32_t',     '0'),
)

xlator_cbks['client_disconnect'] = (
	('fn-arg',      'this',        'xlator_t *'),
	('fn-arg',      'client',      'client_t *'),
	('ret-val',     'int32_t',     '0'),
)

xlator_cbks['ictxmerge'] = (
	('fn-arg',      'this',        'xlator_t *'),
	('fn-arg',      'fd',          'fd_t *'),
	('fn-arg',      'inode',       'inode_t *'),
	('fn-arg',      'linked_inode', 'inode_t *'),
	('ret-val',     'void',        ''),
)

#####################################################################
xlator_dumpops['priv'] = (
	('fn-arg',      'this',        'xlator_t *'),
	('ret-val',     'int32_t',     '0'),
)

xlator_dumpops['inode'] = (
	('fn-arg',      'this',        'xlator_t *'),
	('ret-val',     'int32_t',     '0'),
)

xlator_dumpops['fd'] = (
	('fn-arg',      'this',        'xlator_t *'),
	('ret-val',     'int32_t',     '0'),
)

xlator_dumpops['inodectx'] = (
	('fn-arg',      'this',        'xlator_t *'),
	('fn-arg',      'ino',         'inode_t *'),
	('ret-val',     'int32_t',     '0'),
)

xlator_dumpops['fdctx'] = (
	('fn-arg',      'this',        'xlator_t *'),
	('fn-arg',      'fd',          'fd_t *'),
	('ret-val',     'int32_t',     '0'),
)

xlator_dumpops['priv_to_dict'] = (
	('fn-arg',      'this',        'xlator_t *'),
	('fn-arg',      'dict',        'dict_t *'),
	('ret-val',     'int32_t',     '0'),
)

xlator_dumpops['inode_to_dict'] = (
	('fn-arg',      'this',        'xlator_t *'),
	('fn-arg',      'dict',        'dict_t *'),
	('ret-val',     'int32_t',     '0'),
)

xlator_dumpops['fd_to_dict'] = (
	('fn-arg',      'this',        'xlator_t *'),
	('fn-arg',      'dict',        'dict_t *'),
	('ret-val',     'int32_t',     '0'),
)

xlator_dumpops['inodectx_to_dict'] = (
	('fn-arg',      'this',        'xlator_t *'),
	('fn-arg',      'ino',         'inode_t *'),
	('fn-arg',      'dict',        'dict_t *'),
	('ret-val',     'int32_t',     '0'),
)

xlator_dumpops['fdctx_to_dict'] = (
	('fn-arg',      'this',        'xlator_t *'),
	('fn-arg',      'fd',          'fd_t *'),
	('fn-arg',      'dict',        'dict_t *'),
	('ret-val',     'int32_t',     '0'),
)

xlator_dumpops['history'] = (
	('fn-arg',      'this',        'xlator_t *'),
	('ret-val',     'int32_t',     '0'),
)

def get_error_arg (type_str):
	if type_str.find(" *") != -1:
		return "NULL"
	return "-1"

def get_subs (names, types, cbktypes=None):
	sdict = {}
	sdict["@SHORT_ARGS@"] = ', '.join(names)
	# Convert two separate tuples to one of (name, type) sub-tuples.
	as_tuples = list(zip(types, names))
	# Convert each sub-tuple into a "type name" string.
	as_strings = [' '.join(item) for item in as_tuples]
	# Join all of those into one big string.
	sdict["@LONG_ARGS@"] = ',\n\t'.join(as_strings)
	# So much more readable than string.join(map(string.join,zip(...))))
	sdict["@ERROR_ARGS@"] = ', '.join(list(map(get_error_arg, types)))
	if cbktypes is not None:
		sdict["@CBK_ERROR_ARGS@"] = ', '.join(list(map(get_error_arg, cbktypes)))
	return sdict

def generate (tmpl, name, subs):
	text = tmpl.replace("@NAME@", name)
	if name == "writev":
		# More spurious inconsistency.
		text = text.replace("@UPNAME@", "WRITE")
	elif name == "readv":
		text = text.replace("@UPNAME@", "READ")
	else:
		text = text.replace("@UPNAME@", name.upper())
	for old, new in subs[name].items():
		text = text.replace(old, new)
	# TBD: reindent/reformat the result for maximum readability.
	return text

fop_subs = {}
cbk_subs = {}

for name, args in ops.items():

	# Create the necessary substitution strings for fops.
	arg_names = [ a[1] for a in args if a[0] == 'fop-arg']
	arg_types = [ a[2] for a in args if a[0] == 'fop-arg']
	cbk_types = [ a[2] for a in args if a[0] == 'cbk-arg']
	fop_subs[name] = get_subs(arg_names, arg_types, cbk_types)

	# Same thing for callbacks.
	arg_names = [ a[1] for a in args if a[0] == 'cbk-arg']
	arg_types = [ a[2] for a in args if a[0] == 'cbk-arg']
	cbk_subs[name] = get_subs(arg_names, arg_types)

	# Callers can add other subs to these tables, or even create their
	# own tables, using these same techniques, and then pass the result
	# to generate() which would Do The Right Thing with them.



# -*- coding: utf-8 -*-
#
#  Copyright (c) 2016 Red Hat, Inc. <http://www.redhat.com>
#  This file is part of GlusterFS.
#
#  This file is licensed to you under your choice of the GNU Lesser
#  General Public License, version 3 or any later version (LGPLv3 or
#  later), or the GNU General Public License, version 2 (GPLv2), in all
#  cases as published by the Free Software Foundation.
#
# Converts old style args into new style args

from __future__ import print_function
import sys
from argparse import ArgumentParser
import socket
import os

from syncdutils import GsyncdError
from conf import GLUSTERD_WORKDIR


def gethostbyname(hnam):
    """gethostbyname wrapper"""
    try:
        return socket.gethostbyname(hnam)
    except socket.gaierror:
        ex = sys.exc_info()[1]
        raise GsyncdError("failed to resolve %s: %s" %
                          (hnam, ex.strerror))


def secondary_url(urldata):
    urldata = urldata.replace("ssh://", "")
    host, vol = urldata.split("::")
    vol = vol.split(":")[0]
    return "%s::%s" % (host, vol)


def init_gsyncd_template_conf():
    path = GLUSTERD_WORKDIR + "/geo-replication/gsyncd_template.conf"
    dname = os.path.dirname(path)
    if not os.path.exists(dname):
        try:
            os.mkdir(dname)
        except OSError:
            pass

    if not os.path.exists(path):
        fd = os.open(path, os.O_CREAT | os.O_RDWR)
        os.close(fd)


def init_gsyncd_session_conf(primary, secondary):
    secondary = secondary_url(secondary)
    primary = primary.strip(":")
    secondaryhost, secondaryvol = secondary.split("::")
    secondaryhost = secondaryhost.split("@")[-1]

    # Session Config File
    path = "%s/geo-replication/%s_%s_%s/gsyncd.conf" % (
        GLUSTERD_WORKDIR, primary, secondaryhost, secondaryvol)

    if os.path.exists(os.path.dirname(path)) and not os.path.exists(path):
        fd = os.open(path, os.O_CREAT | os.O_RDWR)
        os.close(fd)


def init_gsyncd_conf(path):
    dname = os.path.dirname(path)
    if not os.path.exists(dname):
        try:
            os.mkdir(dname)
        except OSError:
            pass

    if os.path.exists(dname) and not os.path.exists(path):
        fd = os.open(path, os.O_CREAT | os.O_RDWR)
        os.close(fd)


def upgrade():
    # Create dummy template conf(empty), hack to avoid glusterd
    # fail when it does stat to check the existence.
    init_gsyncd_template_conf()

    inet6 = False
    if "--inet6" in sys.argv:
        inet6 = True

    if "--monitor" in sys.argv:
        # python gsyncd.py --path=/bricks/b1
        # --monitor -c gsyncd.conf
        # --iprefix=/var :gv1
        # --glusterd-uuid=f26ac7a8-eb1b-4ea7-959c-80b27d3e43d0
        # f241::gv2
        p = ArgumentParser()
        p.add_argument("primary")
        p.add_argument("secondary")
        p.add_argument("--glusterd-uuid")
        p.add_argument("-c")
        p.add_argument("--iprefix")
        p.add_argument("--path", action="append")
        pargs = p.parse_known_args(sys.argv[1:])[0]

        # Overwrite the sys.argv after rearrange
        init_gsyncd_session_conf(pargs.primary, pargs.secondary)
        sys.argv = [
            sys.argv[0],
            "monitor",
            pargs.primary.strip(":"),
            secondary_url(pargs.secondary),
            "--local-node-id",
            pargs.glusterd_uuid
        ]
    elif "--status-get" in sys.argv:
        # -c gsyncd.conf --iprefix=/var :gv1 f241::gv2
        # --status-get --path /bricks/b1
        p = ArgumentParser()
        p.add_argument("primary")
        p.add_argument("secondary")
        p.add_argument("-c")
        p.add_argument("--path")
        p.add_argument("--iprefix")
        pargs = p.parse_known_args(sys.argv[1:])[0]

        init_gsyncd_session_conf(pargs.primary, pargs.secondary)

        sys.argv = [
            sys.argv[0],
            "status",
            pargs.primary.strip(":"),
            secondary_url(pargs.secondary),
            "--local-path",
            pargs.path
        ]
    elif "--canonicalize-url" in sys.argv:
        # This can accept multiple URLs and converts each URL to the
        # format ssh://USER@IP:gluster://127.0.0.1:VOLUME
        # This format not used in gsyncd, but added for glusterd compatibility
        p = ArgumentParser()
        p.add_argument("--canonicalize-url", nargs="+")
        pargs = p.parse_known_args(sys.argv[1:])[0]

        for url in pargs.canonicalize_url:
            host, vol = url.split("::")
            host = host.replace("ssh://", "")
            remote_addr = host
            if "@" not in remote_addr:
                remote_addr = "root@" + remote_addr

            user, hname = remote_addr.split("@")

            if not inet6:
                hname = gethostbyname(hname)

            print(("ssh://%s@%s:gluster://127.0.0.1:%s" % (
                user, hname, vol)))

        sys.exit(0)
    elif "--normalize-url" in sys.argv:
        # Adds schema prefix as ssh://
        # This format not used in gsyncd, but added for glusterd compatibility
        p = ArgumentParser()
        p.add_argument("--normalize-url")
        pargs = p.parse_known_args(sys.argv[1:])[0]
        print(("ssh://%s" % secondary_url(pargs.normalize_url)))
        sys.exit(0)
    elif "--config-get-all" in sys.argv:
        #  -c gsyncd.conf --iprefix=/var :gv1 f241::gv2 --config-get-all
        p = ArgumentParser()
        p.add_argument("primary")
        p.add_argument("secondary")
        p.add_argument("-c")
        p.add_argument("--iprefix")
        pargs = p.parse_known_args(sys.argv[1:])[0]

        init_gsyncd_session_conf(pargs.primary, pargs.secondary)

        sys.argv = [
            sys.argv[0],
            "config-get",
            pargs.primary.strip(":"),
            secondary_url(pargs.secondary),
            "--show-defaults",
            "--use-underscore"
        ]
    elif "--verify" in sys.argv and "spawning" in sys.argv:
        # Just checks that able to spawn gsyncd or not
        sys.exit(0)
    elif "--secondaryvoluuid-get" in sys.argv:
        # --secondaryvoluuid-get f241::gv2
        p = ArgumentParser()
        p.add_argument("--secondaryvoluuid-get")
        p.add_argument("-c")
        p.add_argument("--iprefix")
        pargs = p.parse_known_args(sys.argv[1:])[0]
        host, vol = pargs.secondaryvoluuid_get.split("::")

        # Modified sys.argv
        sys.argv = [
            sys.argv[0],
            "voluuidget",
            host,
            vol
        ]
    elif "--config-set-rx" in sys.argv:
        # Not required since default conf is not generated
        # and custom conf generated only when required
        # -c gsyncd.conf --config-set-rx remote-gsyncd
        # /usr/local/libexec/glusterfs/gsyncd . .
        # Touch the gsyncd.conf file and create session
        # directory if required
        p = ArgumentParser()
        p.add_argument("-c", dest="config_file")
        pargs = p.parse_known_args(sys.argv[1:])[0]

        # If not template conf then it is trying to create
        # session config, create a empty file instead
        if pargs.config_file.endswith("gsyncd.conf"):
            init_gsyncd_conf(pargs.config_file)
        sys.exit(0)
    elif "--create" in sys.argv:
        # To update monitor status file
        # --create Created -c gsyncd.conf
        # --iprefix=/var :gv1 f241::gv2
        p = ArgumentParser()
        p.add_argument("--create")
        p.add_argument("primary")
        p.add_argument("secondary")
        p.add_argument("-c")
        p.add_argument("--iprefix")
        pargs = p.parse_known_args(sys.argv[1:])[0]

        init_gsyncd_session_conf(pargs.primary, pargs.secondary)

        # Modified sys.argv
        sys.argv = [
            sys.argv[0],
            "monitor-status",
            pargs.primary.strip(":"),
            secondary_url(pargs.secondary),
            pargs.create
        ]
    elif "--config-get" in sys.argv:
        # -c gsyncd.conf --iprefix=/var :gv1 f241::gv2 --config-get pid-file
        p = ArgumentParser()
        p.add_argument("--config-get")
        p.add_argument("primary")
        p.add_argument("secondary")
        p.add_argument("-c")
        p.add_argument("--iprefix")
        pargs = p.parse_known_args(sys.argv[1:])[0]

        init_gsyncd_session_conf(pargs.primary, pargs.secondary)

        # Modified sys.argv
        sys.argv = [
            sys.argv[0],
            "config-get",
            pargs.primary.strip(":"),
            secondary_url(pargs.secondary),
            "--only-value",
            "--show-defaults",
            "--name",
            pargs.config_get.replace("_", "-")
        ]
    elif "--config-set" in sys.argv:
        # ignore session-owner
        if "session-owner" in sys.argv:
            sys.exit(0)

        # --path=/bricks/b1  -c gsyncd.conf :gv1 f241::gv2
        # --config-set log_level DEBUG
        p = ArgumentParser()
        p.add_argument("primary")
        p.add_argument("secondary")
        p.add_argument("--config-set", action='store_true')
        p.add_argument("name")
        p.add_argument("--value")
        p.add_argument("-c")
        pargs = p.parse_known_args(sys.argv[1:])[0]

        init_gsyncd_session_conf(pargs.primary, pargs.secondary)

        # Modified sys.argv
        sys.argv = [
            sys.argv[0],
            "config-set",
            pargs.primary.strip(":"),
            secondary_url(pargs.secondary),
            "--name=%s" % pargs.name,
            "--value=%s" % pargs.value
        ]
    elif "--config-check" in sys.argv:
        # --config-check georep_session_working_dir
        p = ArgumentParser()
        p.add_argument("--config-check")
        p.add_argument("-c")
        pargs = p.parse_known_args(sys.argv[1:])[0]

        # Modified sys.argv
        sys.argv = [
            sys.argv[0],
            "config-check",
            pargs.config_check.replace("_", "-")
        ]
    elif "--config-del" in sys.argv:
        # -c gsyncd.conf --iprefix=/var :gv1 f241::gv2 --config-del log_level
        p = ArgumentParser()
        p.add_argument("--config-del")
        p.add_argument("primary")
        p.add_argument("secondary")
        p.add_argument("-c")
        p.add_argument("--iprefix")
        pargs = p.parse_known_args(sys.argv[1:])[0]

        init_gsyncd_session_conf(pargs.primary, pargs.secondary)

        # Modified sys.argv
        sys.argv = [
            sys.argv[0],
            "config-reset",
            pargs.primary.strip(":"),
            secondary_url(pargs.secondary),
            pargs.config_del.replace("_", "-")
        ]
    elif "--delete" in sys.argv:
        # --delete -c gsyncd.conf --iprefix=/var
        # --path-list=--path=/bricks/b1  :gv1 f241::gv2
        p = ArgumentParser()
        p.add_argument("--reset-sync-time", action="store_true")
        p.add_argument("--path-list")
        p.add_argument("primary")
        p.add_argument("secondary")
        p.add_argument("--iprefix")
        p.add_argument("-c")
        pargs = p.parse_known_args(sys.argv[1:])[0]

        init_gsyncd_session_conf(pargs.primary, pargs.secondary)

        paths = pargs.path_list.split("--path=")
        paths = ["--path=%s" % x.strip() for x in paths if x.strip() != ""]

        # Modified sys.argv
        sys.argv = [
            sys.argv[0],
            "delete",
            pargs.primary.strip(":"),
            secondary_url(pargs.secondary)
        ]
        sys.argv += paths

        if pargs.reset_sync_time:
            sys.argv.append("--reset-sync-time")

    if inet6:
        # Add `--inet6` as first argument
        sys.argv = [sys.argv[0], "--inet6"] + sys.argv[1:]



# -*- coding: utf-8 -*-

# Copyright (c) 2015 Red Hat, Inc. <http://www.redhat.com/>
# This file is part of GlusterFS.
#
# This file is licensed to you under your choice of the GNU Lesser
# General Public License, version 3 or any later version (LGPLv3 or
# later), or the GNU General Public License, version 2 (GPLv2), in all
# cases as published by the Free Software Foundation.

import os
try:
    from ConfigParser import ConfigParser
except ImportError:
    from configparser import ConfigParser

config = ConfigParser()
config.read(os.path.join(os.path.dirname(os.path.abspath(__file__)),
                         "tool.conf"))


def list_change_detectors():
    return dict(config.items("change_detectors")).keys()


def get_opt(opt):
    return config.get("vars", opt)


def get_change_detector(opt):
    return config.get("change_detectors", opt)
